-- -------------------------------------------------------------------------- --
-- AuctionProfit by kunda                                                     --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- AuctionProfit is a World of Warcraft AddOn that saves information from     --
-- your invoice auction house mails.                                          --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- Features:                                                                  --
-- - saves server, buyer name, item name, auction count and profit from       --
--   invoice auction house mails                                              --
-- - sortable and searchable AuctionProfit window                             --
-- - shows profit in tooltip and under the sellername in the Auction House    --
--   (DISABLED by DEFAULT)                                                    --
-- - shows profit in player tooltip (DISABLED by DEFAULT)                     --
-- - all tooltip displays (AH, AuctionProfit window and Player Tooltip) are   --
--   optional and customizable (ALL DISABLED by DEFAULT)                      --
-- - GUI accessible via Interface Option Menu, LDB or Slash commands to view  --
--   the saved data                                                           --
--   slash commands: /ap - /auctionprofit                                     --
--          example: /ap         : open AuctionProfit GUI                     --
--                   /ap options : open AuctionProfit InterfaceOptionsFrame   --
--                                                                            --
-- AuctionProfit ONLY works if you:                                           --
-- 1.) DO NOT USE addons that auto collect money from your mails!             --
-- 2.) Click on an invoice auction house mail to OPEN it! Click the money!    --
--                                                                            --
-- -------------------------------------------------------------------------- --
--                                                                            --
-- ATTENTION:                                                                 --
-- - You MUST OPEN the invoice auction house mail!!! Without opening the mail --
--   no data is available to save!!!                                          --
-- - Using addons that automatically pulls out money from auction house mails --
--   or mails in general (like Postals 'Open all' or 'shift-click' feature)   --
--   can cause this addon not to save data from this mails.                   --
-- - AuctionProfit saves the localized item names, because it's not possible  --
--   to get an itemID from invoice auction house mails.                       --
--   I don't like localized item names, itemIDs are in all cases better, but  --
--   unless Blizzard is changing the API so it's possible to get the itemID   --
--   from an invoice auction house mail I will not change this.               --
-- - There is no way to get the item amount from invoice auction house mails. --
--   (read above)                                                             --
-- - There is currently no way to delete something from the AP database.      --
-- - Data is saved per server regardless of faction.                          --
-- - profit = buyout - consignment resp. profit = bid - consignment !!!       --
--                                                                            --
-- -------------------------------------------------------------------------- --

-- ---------------------------------------------------------------------------------------------------------------------
AuctionProfit_Options = {} -- SavedVariable options table
APserver = {}              -- SavedVariable server data table
APbuyer = {}               -- SavedVariable buyer data table
APitemName = {}            -- SavedVariable item data table
APlastAuctions = {}        -- SavedVariable last auctions data table

AuctionProfit = CreateFrame("Frame")        -- container
local L = AuctionProfit_Locales             -- localization table

local _G            = _G
local select        = _G.select
local math_floor    = _G.math.floor
local string_format = _G.string.format
local string_gsub   = _G.string.gsub
local string_find   = _G.string.find
local string_sub    = _G.string.sub
local string_len    = _G.string.len
local string_match  = _G.string.match
local string_lower  = _G.string.lower
local tonumber      = _G.tonumber
local tostring      = _G.tostring
local table_sort    = _G.table.sort
local table_insert  = _G.table.insert
local table_wipe    = _G.table.wipe

local StartCheck
local BabyOneMoreTime                       -- prevent multi data save
local AP_sortType = 0                       -- data sort type
local AP_sortOrder = 0                      -- data sort order: 0 = asc | 1 = desc
local AP_sortData = 0                       -- data type: 0 = name | 1 = buys/name(last20) | 2 = profit
local AP_LastXAuctions = 20
local AP_ItemNames = {}                     -- table with pure item names from actual server
local AP_ItemIndex = {}                     -- table with item index keys from actual server
local AP_ItemCount = 0                      -- totalnumber of items from actual server
local AP_NameIndex = {}                     -- table with pure names from actual server
local AP_NameCount = 0                      -- totalnumber of names from actual server
local AP_ServerName = GetCVar("realmName")
local AP_ServerNameIndex
local SearchQuery
local dbCount
local AP_ListData

local colorWhite       = "ffffff"           -- {r = 1,      g = 1,      b = 1,      a = 1}
local colorBlue        = "6699ff"           -- {r = 0.4,    g = 0.6,    b = 1,      a = 1}
local colorGreen       = "00b27b"           -- {r = 0.0001, g = 0.6981, b = 0.4824, a = 1}
local colorYellow      = "ffff00"           -- {r = 1,      g = 1,      b = 0,      a = 1}
local colorLightyellow = "ffff7f"           -- {r = 1,      g = 1,      b = 0.4981, a = 1}
local colorGold        = "ffd100"
local colorSilver      = "e6e6e6"
local colorCopper      = "c8602c"

local info = {}                             -- table for InterfaceOptions dropdown menu
local ttline = {
	[1] = L["Name"].."/"..L["Item"].." : "..L["Auctions"].." : "..L["Profit"],
	[2] = L["Auctions"].." : "..L["Name"].."/"..L["Item"].." : "..L["Profit"],
	[3] = L["Auctions"].." : "..L["Profit"].." : "..L["Name"].."/"..L["Item"],
	[4] = L["Profit"].." : "..L["Name"].."/"..L["Item"].." : "..L["Auctions"],
	[5] = L["Profit"].." : "..L["Auctions"].." : "..L["Name"].."/"..L["Item"]
}
local ttpos = {
	ANCHOR_TOPRIGHT    = L["BottomRight Tooltip corner -> anchored to TopRight Line corner"],
	ANCHOR_RIGHT       = L["BottomLeft Tooltip corner -> anchored to TopRight Line corner"],
	ANCHOR_BOTTOMRIGHT = L["TopLeft Tooltip corner -> anchored to BottomRight Line corner"],
	ANCHOR_TOPLEFT     = L["BottomLeft Tooltip corner -> anchored to TopLeft Line corner"],
	ANCHOR_LEFT        = L["BottomRight Tooltip corner -> anchored to TopLeft Line corner"],
	ANCHOR_BOTTOMLEFT  = L["TopRight Tooltip corner -> anchored to BottomLeft Line corner"]
}

-- local functions
local function Print(...)
	print("|cff"..colorLightyellow.."AuctionProfit:|r", ...)
end

local function moneyFormat(money)
	local gold = math_floor(money/10000)
	local silver = math_floor((money-gold*10000)/100)
	local copper = money-gold*10000-silver*100
	local m
	if money < 100 then
		m = "|cff"..colorCopper..copper..L["c"].."|r"
	end
	if money >= 100 and money < 10000 then
		if copper < 10 then copper = "0"..copper end
		m = "|cff"..colorSilver..silver..L["s"].."|r |cff"..colorCopper..copper..L["c"].."|r"
	end
	if money >= 10000 then
		if copper < 10 then copper = "0"..copper end
		if silver < 10 then silver = "0"..silver end
		m = "|cff"..colorGold..gold..L["g"].."|r |cff"..colorSilver..silver..L["s"].."|r |cff"..colorCopper..copper..L["c"].."|r"
	end
	return m
end

-- parameter: x = string | y = number (longest string lenght)
-- example: x = "193" | y = 8
-- return: string
--         (example: "00000bjd")
--
-- matrix: x0123456789
--         0abcdefghij
local function numMatrix(x, y)
	local xl = string_len(x)
	local xd
	x = string_gsub(x, "0", "a")
	x = string_gsub(x, "1", "b")
	x = string_gsub(x, "2", "c")
	x = string_gsub(x, "3", "d")
	x = string_gsub(x, "4", "e")
	x = string_gsub(x, "5", "f")
	x = string_gsub(x, "6", "g")
	x = string_gsub(x, "7", "h")
	x = string_gsub(x, "8", "i")
	x = string_gsub(x, "9", "j")
	if xl < y then
		xd = y - xl
		for i=1, xd do		
			x = "0"..x
		end
	end
	return x
end

-- parameter: str = string | part = number
-- example: str = "PlayerName:211375:8" | part = 2
-- return: string
--         (example: "211375")
--
-- xxxxxxxxxx:xxxxxx:x
-- 1          2      3
-- PlayerName:211375:8
local function strReturn(str, part)
	local part1, part2, part3 = string_match(str, "(.+):(.+):(.+)")
	if     part == 3 then return part3
	elseif part == 2 then return part2
	elseif part == 0 then return part1, part2, part3
	elseif part == 1 then return part1 end
end

-- parameter: x = string
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1" 
-- return: string
--         (example: "PlayerName")
local function buyerNameReturn(x)
	local pos = string_find(x, ":", 1)
	local str = string_sub(x, 1, pos-1)
	return str
end

-- parameter: x = string
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
-- return: string
--         (example: "PlayerName:211375:8")
local function buyerMainDataReturn(x)
	local pos = string_find(x, "#", 1)
	local str = string_sub(x, 1, pos-1)
	return str
end

-- parameter: x = string
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
-- return: string
--         (example: "15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1")
local function buyerSubDataReturn(x)
	local l = string_len(x)
	local pos = string_find(x, "#", 1)
	local str = string_sub(x, pos+1, l)
	return str
end

-- parameter: x = string
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
-- return: number
--         n (example: return 5 for 5 SubData entries: 1="15-129675-4"  2="20-13300-1"  3="77-31350-1"  4="42-22800-1"  5="10-14250-1")
local function buyerSubDataCountReturn(x)
	local a, b = string_gsub(buyerSubDataReturn(x), ":", ":")
	return (b+1)
end

-- parameter: x = string | y = number
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
--          y = itemID
-- return: number
--         0 = no itemID avaiable  1 = itemID availabe
local function buyerSubDataItemIDcheckReturn(x, y)
	local l = string_len(x)
	local pos = string_find(x,"#",1)
	local a = ":"..string_sub(x,pos+1,l)
	local b = ":"..y.."-"
	if string_find(a,b,1,true) then
		return true
	end
end

-- parameter: x = string | y = number
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
--          y = itemID
-- return: string
--         (example: part from y (itemID) --> y=77 return:"77-31350-1")
local function buyerSubDataPartReturn(x, y)
	local a = ":"..buyerSubDataReturn(x)..":"
	local b = ":"..tostring(y).."-"
	local pos = string_find(a, b, 1, true)
	local l = string_len(a)
	local r = string_sub(a, pos+1, l)
	local rpos = string_find(r, ":", 1)
	local part = string_sub(r, 1, rpos-1)
	return part
end

-- parameter: x = string | y = number
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
--          y = itemID
-- return: string
--         (example: profit from y (itemID) --> y=77 return:"31350")
local function buyerSubDataPartProfitReturn(x, y)
	local a = buyerSubDataPartReturn(x, y)
	local l = string_len(a)
	local pos = string_find(a, "-", 1, true)
	local pos2 = string_find(a, "-", (l*-1)+pos)
	local r = string_sub(a, pos+1, pos2-1)
	return r
end

-- parameter: x = string | y = number
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
--          y = itemID
-- return: string
--         (example: buys from y (itemID) --> y=15 return:"4")
local function buyerSubDataPartBuysReturn(x, y)
	local a = buyerSubDataPartReturn(x, y)
	local l = string_len(a)
	local pos = string_find(a, "-", 1, true)
	local pos2 = string_find(a, "-", (l*-1)+pos)
	local r = string_sub(a, pos2+1, l)
	return r
end

-- parameter: x = string | y = string (replace) | z = string (new)
-- example: x = "PlayerName:211375:8#15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1"
--          y = "15-129675-4"
--          z = "15-176483-5"
-- return: string
--         (example: old string x      "15-129675-4:20-13300-1:77-31350-1:42-22800-1:10-14250-1")
--         (         new return string "15-176483-5:20-13300-1:77-31350-1:42-22800-1:10-14250-1")
local function buyerSubDataPartReplaceReturn(x, y, z)
	local x1 = buyerSubDataReturn(x)
	local y1 = string_gsub(y, "-", "%%-")
	local r = string_gsub(x1, y1, z)
	return r
end

-- parameter: x = string
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6"
-- return: string
--         (example: return "Begrenzter Unverwundbarkeitstrank")
local function itemNameReturn(x)
	local pos = string_find(x, "#", 1)
	local str = string_sub(x, 1, pos-1)
	return str
end

-- parameter: x = string
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6:2-56782-4"
-- return: string
--         (example: return "1-83500-6:2-56782-4")
local function itemNameSubDataReturn(x)
	local l = string_len(x)
	local pos = string_find(x, "#", 1)
	local str = string_sub(x, pos+1, l)
	return str
end

-- parameter: x = string | y = number (serverID)
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6:2-56782-4"
--          y = 2
-- return: number 
--         (example: return 1 - 0 = serverID not exists  1 = serverID exists)
local function itemNameServerCheckReturn(x, y)
	local a = ":"..itemNameSubDataReturn(x)..":"
	local b = ":"..tostring(y).."-"
	if string_find(a, b, 1, true) then
		return true
	end
end

-- parameter: x = string | y = number (serverID)
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6:2-56782-4"
--          y = 2
-- return: string
--         (example: part von y (serverID) --> y=2 return:"2-56782-4")
local function itemNameSubDataPartReturn(x, y)
	local a = ":"..itemNameSubDataReturn(x)..":" -- a = :1-83500-6:2-56782-4:
	local b = ":"..tostring(y).."-"
	local pos = string_find(a, b, 1, true)
	local l = string_len(a)
	local r = string_sub(a, pos+1, l)
	local rpos = string_find(r, ":", 1)
	local part = string_sub(r, 1, rpos-1)
	return part
end

-- parameter: x = string | y = number (serverID)
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6:2-56782-4"
--          y = 2 
-- return: string
--         (example: profit von y (serverID) --> y=2 return:"56782")
local function itemNameSubDataPartProfitReturn(x, y)
	local a = itemNameSubDataPartReturn(x, y)
	local l = string_len(a)
	local pos = string_find(a, "-", 1, true)
	local pos2 = string_find(a, "-", (l*-1)+pos)
	local r = string_sub(a, pos+1, pos2-1)
	return r
end

-- parameter: x = string | y = number (serverID)
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6:2-56782-4"
--          y = 2 
-- return: string
--         (example: buys von y (serverID) --> y=2 return:"4")
local function itemNameSubDataPartBuysReturn(x, y)
	local a = itemNameSubDataPartReturn(x, y)
	local l = string_len(a)
	local pos = string_find(a, "-", 1, true)
	local pos2 = string_find(a, "-", (l*-1)+pos)
	local r = string_sub(a, pos2+1, l)
	return r
end

-- parameter: x = string | y = string (replace) | z = string (new)
-- example: x = "Begrenzter Unverwundbarkeitstrank#1-83500-6:2-56782-4"
--          y = "1-83500-6"
--          z = "1-117540-7"
-- return: string
--         (example: old string x      "1-83500-6:2-56782-4")
--         (example: new return string "1-117540-7:2-56782-4")
local function itemNameSubDataPartReplaceReturn(x, y, z)
	local x1 = itemNameSubDataReturn(x)
	local y1 = string_gsub(y, "-", "%%-")
	local r = string_gsub(x1, y1, z)
	return r
end

-- parameter: x = string (date#ItemID:Profit:PlayerNameID) | s = number (serverID)
-- example: date = date("%y%m%d %H%M")
--          x = "071126 1922#23:11400:460"
--          s = 1
-- returns: string
--          (example: "|cffffffff071126 1922|r someitem:11400:somename |cffffffff(count)")
--
-- example: x = "date#ItemID:Profit:PlayerNameID"
--          s = 1
--          return: "date ItemName:Profit:PlayerName (count)"
local function lastAuctionsReturn(x, s)
	local pos = string_find(x, "#", 1)
	local xdate = string_sub(x, 1, pos-1)

	local x = string_sub(x, pos+1)

	-- a1 = itemID | a2 = profit | a3 = playerNameID
	local a1, a2, a3 = strReturn(x, 0)

	local Buyer = APbuyer[s][tonumber(a3)]
	local MainData = buyerMainDataReturn(Buyer)
	local a3buys = strReturn(MainData, 3)
	local a3profit = strReturn(MainData, 2)

	local a1r = itemNameReturn(APitemName[tonumber(a1)]) -- return itemName
	local a3r = buyerNameReturn(Buyer) -- return playerName

	local r = "|cff"..colorWhite..xdate.."|r "..a1r..":"..tostring(a2)..":"..a3r.." |cff"..colorWhite.."("..tostring(a3buys)..")|r"
	return r
end

local function setHeadlineTabButton(name, showIt)
	if showIt then
		local button = _G[name]
		button:SetNormalFontObject("GameFontWhite")
		button:Disable()
		local border = _G[name.."Border"]
		border:SetPoint("TOPLEFT"     ,1, -1)
		border:SetPoint("BOTTOMLEFT"  ,1, 0)
		border:SetPoint("TOPRIGHT"    ,-1, -1)
		border:SetPoint("BOTTOMRIGHT" ,-1, 0)
		_G[name.."TempColor"]:SetTexture(0.4, 0.4, 0.4, 1)
	else
		local button = _G[name]
		button:SetNormalFontObject("GameFontNormal")
		button:Enable()
		local border = _G[name.."Border"]
		border:SetPoint("TOPLEFT"     ,1, -1)
		border:SetPoint("BOTTOMLEFT"  ,0, 1)
		border:SetPoint("TOPRIGHT"    ,-1, -1)
		border:SetPoint("BOTTOMRIGHT" ,-1, 1)
		_G[name.."TempColor"]:SetTexture(0.4, 0.4, 0.4, 0.5)
	end
end

local function AuctionProfit_OopsIDidItAgain()
	BabyOneMoreTime = nil
end

local function AuctionProfit_TakeInboxMoney(index)
	if not index then return end
	if BabyOneMoreTime then return end
	BabyOneMoreTime = true
	
	local invoiceType, itemName, playerName, bid, buyout, deposit, consignment = GetInboxInvoiceInfo(index)

	if invoiceType and invoiceType == "seller" then
		AuctionProfit:SaveData(invoiceType, itemName, playerName, bid, buyout, deposit, consignment)
	end
end

local function SlashHandler(msg)
	msg = strtrim(string.lower(msg or ""))
	if msg == "options" then
		InterfaceOptionsFrame_OpenToCategory("AuctionProfit")
	else
		AuctionProfit:FrameToggle()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:InitOptions()
	SlashCmdList["AUCTIONPROFIT"] = SlashHandler
	SLASH_AUCTIONPROFIT1 = "/ap"
	SLASH_AUCTIONPROFIT2 = "/auctionprofit"

	if AuctionProfit_Options.version == nil                  then AuctionProfit_Options.version                  = 2.1   end

	if AuctionProfit_Options.ChatOutput == nil               then AuctionProfit_Options.ChatOutput               = true  end
	if AuctionProfit_Options.TooltipLineDisplay == nil       then AuctionProfit_Options.TooltipLineDisplay       = 5     end
	if AuctionProfit_Options.TooltipItemNameLength == nil    then AuctionProfit_Options.TooltipItemNameLength    = 40    end

	if AuctionProfit_Options.PlayerTooltipToggle == nil      then AuctionProfit_Options.PlayerTooltipToggle      = false end
	if AuctionProfit_Options.PlayerTooltipOOC == nil         then AuctionProfit_Options.PlayerTooltipOOC         = true  end
	if AuctionProfit_Options.PlayerTooltipLines == nil       then AuctionProfit_Options.PlayerTooltipLines       = 10    end

	if AuctionProfit_Options.AHBrowseTabProfit == nil        then AuctionProfit_Options.AHBrowseTabProfit        = false end
	if AuctionProfit_Options.AHTooltipToggle == nil          then AuctionProfit_Options.AHTooltipToggle          = false end
	if AuctionProfit_Options.AHTooltipLines == nil           then AuctionProfit_Options.AHTooltipLines           = 20    end
	if AuctionProfit_Options.AHTooltipPosAnchor == nil       then AuctionProfit_Options.AHTooltipPosAnchor       = "ANCHOR_BOTTOMRIGHT" end
	if AuctionProfit_Options.AHTooltipPosX == nil            then AuctionProfit_Options.AHTooltipPosX            = -250  end
	if AuctionProfit_Options.AHTooltipPosY == nil            then AuctionProfit_Options.AHTooltipPosY            = 0     end

	if AuctionProfit_Options.APwindowSearchShow == nil       then AuctionProfit_Options.APwindowSearchShow       = false end
	if AuctionProfit_Options.APwindowToggle == nil           then AuctionProfit_Options.APwindowToggle           = false end
	if AuctionProfit_Options.APwindowTooltipToggle == nil    then AuctionProfit_Options.APwindowTooltipToggle    = false end
	if AuctionProfit_Options.APwindowTooltipLines == nil     then AuctionProfit_Options.APwindowTooltipLines     = 20    end
	if AuctionProfit_Options.APwindowTooltipPosAnchor == nil then AuctionProfit_Options.APwindowTooltipPosAnchor = "ANCHOR_BOTTOMRIGHT" end
	if AuctionProfit_Options.APwindowTooltipPosX == nil      then AuctionProfit_Options.APwindowTooltipPosX      = -250  end
	if AuctionProfit_Options.APwindowTooltipPosY == nil      then AuctionProfit_Options.APwindowTooltipPosY      = -5    end

	if AuctionProfit_Options.StartMessage == nil             then AuctionProfit_Options.StartMessage             = true  end

	-- one time start message
	if AuctionProfit_Options.StartMessage then
		Print(L["Login WARNING"])
		Print(L["1.) DO NOT USE addons that auto collect money from your mails!"])
		Print(L["2.) Click on an invoice auction house mail to OPEN it! Click the money!"])
		AuctionProfit_Options.StartMessage = false
	end

	-- Auctioneer UI check - NO Auctioneer Advanced UI support, because Auctioneer Advanced UI makes a mess with the default AH Interface.
	-- -> AuctionProfit self-castration to avoid errors
	--    If you want the Auction House features from AuctionProfit: disable Auctioneer Advanced UI!
	if IsAddOnLoaded("Auc-Advanced") and AucAdvanced.Settings.GetSetting("util.compactui.activated") then
		AuctionProfit:UnregisterEvent("AUCTION_HOUSE_SHOW")
		--Print("Auc-Advanced found - AuctionProfit self-castration = NO AP Auction House feature will work!")
		--Print("If you want the Auction House features from AuctionProfit: disable Auctioneer Advanced UI!")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:LDBcheck()
	if LibStub and LibStub:GetLibrary("CallbackHandler-1.0", true) and LibStub:GetLibrary("LibDataBroker-1.1", true) then
		LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("AuctionProfit", {
			type = "launcher",
			icon = "Interface\\GroupFrame\\UI-Group-MasterLooter",
			OnClick = function(self, button)
				AuctionProfit:FrameToggle()
			end,
		})
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:CreateInterfaceOptions()
	-- panel General START
	local options = CreateFrame("FRAME", "AuctionProfit_InterfaceOptions")
	options.name = "AuctionProfit"

	local title = options:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("AuctionProfit")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)

	local subText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subText:SetText(L["AuctionProfit saves information from your invoice auction house mails."])
	subText:SetNonSpaceWrap(true)
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subText:SetWidth(350)--TODO

	local GUI = CreateFrame("Button", "AuctionProfit_InterfaceOptions_GUI", AuctionProfit_InterfaceOptions, "UIPanelButtonTemplate2")
	AuctionProfit_InterfaceOptions_GUI:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 0, -10)
	AuctionProfit_InterfaceOptions_GUI:SetWidth(120)
	AuctionProfit_InterfaceOptions_GUI:SetHeight(24)
	AuctionProfit_InterfaceOptions_GUIText:SetText(L["AuctionProfit Window"])
	AuctionProfit_InterfaceOptions_GUI:SetWidth( AuctionProfit_InterfaceOptions_GUIText:GetWidth()+20 )
	AuctionProfit_InterfaceOptions_GUI:SetScript("OnClick", function(self)
		InterfaceOptionsFrame_Show() -- hide InterfaceOptionsFrame
		HideUIPanel(GameMenuFrame)   -- hide GameMenuFrame
		AuctionProfit:FrameToggle()  -- show AuctionProfit GUI
	end)

	local slashcommandText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	slashcommandText:SetText("(|cff"..colorLightyellow.."/ap|r - |cff"..colorLightyellow.."/auctionprofit|r)")
	slashcommandText:SetNonSpaceWrap(true)
	slashcommandText:SetPoint("LEFT", GUI, "RIGHT", 10, 0)

	local titleGeneral = options:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleGeneral:SetPoint("TOPLEFT", GUI, "BOTTOMLEFT", 0, -10)
	titleGeneral:SetText(L["General"])
	titleGeneral:SetNonSpaceWrap(true)
	titleGeneral:SetJustifyH("LEFT")
	titleGeneral:SetJustifyV("TOP")

	local ChatMessage = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptions_ChatMessage", AuctionProfit_InterfaceOptions, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptions_ChatMessage:SetPoint("TOPLEFT", titleGeneral, "TOPLEFT", 0, -20)
	AuctionProfit_InterfaceOptions_ChatMessageText:SetText(L["Chat Output"])
	AuctionProfit_InterfaceOptions_ChatMessage:SetChecked(AuctionProfit_Options.ChatOutput)
	AuctionProfit_InterfaceOptions_ChatMessage:SetScript("OnClick", function(self)
		AuctionProfit_Options.ChatOutput = not AuctionProfit_Options.ChatOutput
	end)
	ChatMessage.tooltipText = L["Shows chat message when new data was saved."]

	local TooltipLine = CreateFrame("Frame", "AuctionProfit_InterfaceOptions_TooltipLine", AuctionProfit_InterfaceOptions, "UIDropDownMenuTemplate")
	AuctionProfit_InterfaceOptions_TooltipLine:SetPoint("TOPLEFT", ChatMessage, "TOPLEFT", -16, -50)
	AuctionProfit_InterfaceOptions_TooltipLine:CreateFontString("AuctionProfit_InterfaceOptions_TooltipLineLabel", "BACKGROUND", "GameFontHighlight")
	AuctionProfit_InterfaceOptions_TooltipLineLabel:SetPoint("BOTTOMLEFT", AuctionProfit_InterfaceOptions_TooltipLine, "TOPLEFT", 16, 5)
	AuctionProfit_InterfaceOptions_TooltipLineLabel:SetText(L["Tooltip Line Appearance"]..":")
	AuctionProfit_InterfaceOptions_TooltipLine.initialize = function()
		for i=1, #ttline do
			info.text = ttline[i]
			info.arg1 = i
			info.arg2 = nil
			info.func = function(button, selected)
				AuctionProfit_Options.TooltipLineDisplay = selected
				AuctionProfit_InterfaceOptions_TooltipLineText:SetText(ttline[AuctionProfit_Options.TooltipLineDisplay])
				UIDropDownMenu_SetWidth(AuctionProfit_InterfaceOptions_TooltipLine, AuctionProfit_InterfaceOptions_TooltipLineText:GetWidth()+25)
			end
			if AuctionProfit_Options.TooltipLineDisplay == info.arg1 then
				info.checked = 1
			else
				info.checked = nil
			end
			info.keepShownOnClick = nil
			UIDropDownMenu_AddButton(info)
		end
	end
	AuctionProfit_InterfaceOptions_TooltipLineText:SetText(ttline[AuctionProfit_Options.TooltipLineDisplay])
	UIDropDownMenu_SetWidth(AuctionProfit_InterfaceOptions_TooltipLine, AuctionProfit_InterfaceOptions_TooltipLineText:GetWidth()+25)
	UIDropDownMenu_EnableDropDown(AuctionProfit_InterfaceOptions_TooltipLine)

	local TooltipLineInfoText = options:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	TooltipLineInfoText:SetPoint("TOPLEFT", TooltipLine, "BOTTOMLEFT", 24, 0)
	TooltipLineInfoText:SetText(L["(All Tooltip displays!)"])
	TooltipLineInfoText:SetNonSpaceWrap(true)
	TooltipLineInfoText:SetJustifyV("CENTER")
	-- panel General END

	-- panel Player START
	local optionsPlayer = CreateFrame("FRAME", "AuctionProfit_InterfaceOptionsPlayer")
	optionsPlayer.name = L["Player"]
	optionsPlayer.parent = "AuctionProfit"

	local title = optionsPlayer:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("AuctionProfit")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)
	
	local titlePlayer = optionsPlayer:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titlePlayer:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	titlePlayer:SetText(L["Player"])
	titlePlayer:SetNonSpaceWrap(true)
	titlePlayer:SetJustifyH("LEFT")
	titlePlayer:SetJustifyV("TOP")
	
	local PlayerTTtoggle = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptionsPlayer_PlayerTTtoggle", AuctionProfit_InterfaceOptionsPlayer, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTtoggle:SetPoint("TOPLEFT", titlePlayer, "TOPLEFT", 0, -20)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTtoggleText:SetText(L["Enable/Disable Tooltip"])
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTtoggle:SetChecked(AuctionProfit_Options.PlayerTooltipToggle)
	PlayerTTtoggle.tooltipText = L["Enable or disable AuctionProfit data in player tooltip."]

	local PlayerTTOOC = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC", AuctionProfit_InterfaceOptionsPlayer, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC:SetPoint("TOPLEFT", PlayerTTtoggle, "TOPLEFT", 20, -25)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOCText:SetText(L["Tooltip Out-Of-Combat only"])
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC:SetChecked(AuctionProfit_Options.PlayerTooltipOOC)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC:SetScript("OnClick", function(self)
		AuctionProfit_Options.PlayerTooltipOOC = not AuctionProfit_Options.PlayerTooltipOOC
	end)
	PlayerTTOOC.tooltipText = L["Show AuctionProfit data in player tooltip only if you are OOC (Out-Of-Combat)."]

	local PlayerTTlines = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines", AuctionProfit_InterfaceOptionsPlayer, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetWidth(200)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetHeight(16)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetPoint("TOPLEFT", PlayerTTOOC, "TOPLEFT", 8, -40)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlinesText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlinesText:SetText(L["Tooltip Lines"]..": "..AuctionProfit_Options.PlayerTooltipLines)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlinesLow:SetText("0")
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlinesHigh:SetText("40")
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetMinMaxValues(0, 40)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetValueStep(1)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetValue(AuctionProfit_Options.PlayerTooltipLines)
	AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsPlayer_PlayerTTlinesText:SetText(L["Tooltip Lines"]..": "..value)
		AuctionProfit_Options.PlayerTooltipLines = value
	end)
	PlayerTTlines.tooltipText = L["Maximum Lines in Player Tooltip.\n0 = Only Profit"]

	if AuctionProfit_Options.PlayerTooltipToggle then
		BlizzardOptionsPanel_CheckButton_Enable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC, true)
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines)
	else
		BlizzardOptionsPanel_CheckButton_Disable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC)
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines)
	end

	AuctionProfit_InterfaceOptionsPlayer_PlayerTTtoggle:SetScript("OnClick", function(self)
		AuctionProfit_Options.PlayerTooltipToggle = not AuctionProfit_Options.PlayerTooltipToggle
		if AuctionProfit_Options.PlayerTooltipToggle then
			BlizzardOptionsPanel_CheckButton_Enable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC, true)
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines)
		else
			BlizzardOptionsPanel_CheckButton_Disable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTOOC)
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsPlayer_PlayerTTlines)
		end
	end)
	-- panel Player END

	-- panel AuctionProfit Window START
	local optionsAPframe = CreateFrame("FRAME", "AuctionProfit_InterfaceOptionsAPframe")
	optionsAPframe.name = L["AuctionProfit Window"]
	optionsAPframe.parent = "AuctionProfit"

	local title = optionsAPframe:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("AuctionProfit")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)
	
	local titleAPframe = optionsAPframe:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleAPframe:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	titleAPframe:SetText(L["AuctionProfit Window"])
	titleAPframe:SetNonSpaceWrap(true)
	titleAPframe:SetJustifyH("LEFT")
	titleAPframe:SetJustifyV("TOP")

	local APframeAutoToggle = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptionsAPframe_APframeAutoToggle", AuctionProfit_InterfaceOptionsAPframe, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptionsAPframe_APframeAutoToggle:SetPoint("TOPLEFT", titleAPframe, "TOPLEFT", 0, -20)
	AuctionProfit_InterfaceOptionsAPframe_APframeAutoToggleText:SetText(L["Auto show AuctionProfit Window"])
	AuctionProfit_InterfaceOptionsAPframe_APframeAutoToggle:SetChecked(AuctionProfit_Options.APwindowToggle)
	AuctionProfit_InterfaceOptionsAPframe_APframeAutoToggle:SetScript("OnClick", function(self)
		AuctionProfit_Options.APwindowToggle = not AuctionProfit_Options.APwindowToggle
	end)
	APframeAutoToggle.tooltipText = L["Automatically show AuctionProfit Window when new data from an invoice auction house mail is saved."]

	local APframeTooltip = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptionsAPframe_APframeTooltip", AuctionProfit_InterfaceOptionsAPframe, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptionsAPframe_APframeTooltip:SetPoint("TOPLEFT", APframeAutoToggle, "TOPLEFT", 0, -20)
	AuctionProfit_InterfaceOptionsAPframe_APframeTooltipText:SetText(L["Enable/Disable Tooltip"])
	AuctionProfit_InterfaceOptionsAPframe_APframeTooltip:SetChecked(AuctionProfit_Options.APwindowTooltipToggle)
	APframeTooltip.tooltipText = L["Enable or disable AuctionProfit data in AuctionProfit Window tooltip."]

	local APframeTTlines = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsAPframe_APframeTTlines", AuctionProfit_InterfaceOptionsAPframe, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetWidth(200)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetHeight(16)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetPoint("TOPLEFT", APframeTooltip, "TOPLEFT", 28, -40)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlinesText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlinesText:SetText(L["Tooltip Lines"]..": "..AuctionProfit_Options.APwindowTooltipLines)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlinesLow:SetText("1")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlinesHigh:SetText("40")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetMinMaxValues(1, 40)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetValueStep(1)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetValue(AuctionProfit_Options.APwindowTooltipLines)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTlines:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsAPframe_APframeTTlinesText:SetText(L["Tooltip Lines"]..": "..value)
		AuctionProfit_Options.APwindowTooltipLines = value
	end)
	APframeTTlines.tooltipText = L["Maximum Lines in AuctionProfit Window Tooltip."]

	local APframeTTposition = CreateFrame("Frame", "AuctionProfit_InterfaceOptionsAPframe_APframeTTposition", AuctionProfit_InterfaceOptionsAPframe, "UIDropDownMenuTemplate")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposition:SetPoint("TOPLEFT", APframeTTlines, "TOPLEFT", -16, -50)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposition:CreateFontString("AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionLabel", "BACKGROUND", "GameFontHighlight")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionLabel:SetPoint("BOTTOMLEFT", AuctionProfit_InterfaceOptionsAPframe_APframeTTposition, "TOPLEFT", 16, 5)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionLabel:SetText(L["Tooltip Position"]..":")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposition.initialize = function()
		for k,v in pairs(ttpos) do
			info.text = k
			info.arg1 = k
			info.arg2 = nil
			info.func = function(button, selected)
				AuctionProfit_Options.APwindowTooltipPosAnchor = selected
				AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionText:SetText(AuctionProfit_Options.APwindowTooltipPosAnchor)
				UIDropDownMenu_SetWidth(AuctionProfit_InterfaceOptionsAPframe_APframeTTposition, AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionText:GetWidth()+25)
			end
			if AuctionProfit_Options.APwindowTooltipPosAnchor == info.arg1 then
				info.checked = 1
			else
				info.checked = nil
			end
			info.keepShownOnClick = nil
			info.tooltipTitle = k
			info.tooltipText = v
			UIDropDownMenu_AddButton(info)
		end
	end
	AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionText:SetText(AuctionProfit_Options.APwindowTooltipPosAnchor)
	UIDropDownMenu_SetWidth(AuctionProfit_InterfaceOptionsAPframe_APframeTTposition, AuctionProfit_InterfaceOptionsAPframe_APframeTTpositionText:GetWidth()+25)

	local APframeTTposX = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsAPframe_APframeTTposX", AuctionProfit_InterfaceOptionsAPframe, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetWidth(200)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetHeight(16)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetPoint("TOPLEFT", APframeTTposition, "TOPLEFT", 16, -50)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposXText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposXText:SetText(L["Pos X"]..": "..AuctionProfit_Options.APwindowTooltipPosX)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposXLow:SetText("-600")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposXHigh:SetText("600")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetMinMaxValues(-600, 600)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetValueStep(10)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetValue(AuctionProfit_Options.APwindowTooltipPosX)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposX:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsAPframe_APframeTTposXText:SetText(L["Pos X"]..": "..value)
		AuctionProfit_Options.APwindowTooltipPosX = value
	end)
	APframeTTposX.tooltipText = L["Pos X"]

	local APframeTTposY = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsAPframe_APframeTTposY", AuctionProfit_InterfaceOptionsAPframe, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetWidth(200)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetHeight(16)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetPoint("TOPLEFT", APframeTTposX, "TOPLEFT", 0, -50)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposYText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposYText:SetText(L["Pos X"]..": "..AuctionProfit_Options.APwindowTooltipPosY)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposYLow:SetText("-50")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposYHigh:SetText("50")
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetMinMaxValues(-50, 50)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetValueStep(5)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetValue(AuctionProfit_Options.APwindowTooltipPosY)
	AuctionProfit_InterfaceOptionsAPframe_APframeTTposY:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsAPframe_APframeTTposYText:SetText(L["Pos Y"]..": "..value)
		AuctionProfit_Options.APwindowTooltipPosY = value
	end)
	APframeTTposY.tooltipText = L["Pos Y"]

	if AuctionProfit_Options.APwindowTooltipToggle then
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAPframe_APframeTTlines)
		UIDropDownMenu_EnableDropDown(AuctionProfit_InterfaceOptionsAPframe_APframeTTposition)
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposX)
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposY)
	else
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAPframe_APframeTTlines)
		UIDropDownMenu_DisableDropDown(AuctionProfit_InterfaceOptionsAPframe_APframeTTposition)
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposX)
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposY)
	end

	AuctionProfit_InterfaceOptionsAPframe_APframeTooltip:SetScript("OnClick", function(self)
		AuctionProfit_Options.APwindowTooltipToggle = not AuctionProfit_Options.APwindowTooltipToggle
		if AuctionProfit_Options.APwindowTooltipToggle then
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAPframe_APframeTTlines)
			UIDropDownMenu_EnableDropDown(AuctionProfit_InterfaceOptionsAPframe_APframeTTposition)
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposX)
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposY)
		else
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAPframe_APframeTTlines)
			UIDropDownMenu_DisableDropDown(AuctionProfit_InterfaceOptionsAPframe_APframeTTposition)
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposX)
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAPframe_APframeTTposY)
		end
	end)
	-- panel AuctionProfit window END

	-- panel Auction House START
	local optionsAH = CreateFrame("FRAME", "AuctionProfit_InterfaceOptionsAH")
	optionsAH.name = L["Auction House"]
	optionsAH.parent = "AuctionProfit"

	local title = optionsAH:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("AuctionProfit")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)
	
	local titleAH = optionsAH:CreateFontString(nil, "ARTWORK", "GameFontNormal")
	titleAH:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	titleAH:SetText(L["Auction House"])
	titleAH:SetNonSpaceWrap(true)
	titleAH:SetJustifyH("LEFT")
	titleAH:SetJustifyV("TOP")

	local AHbrowseProfit = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptionsAH_AHbrowseProfit", AuctionProfit_InterfaceOptionsAH, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptionsAH_AHbrowseProfit:SetPoint("TOPLEFT", titleAH, "TOPLEFT", 0, -20)
	AuctionProfit_InterfaceOptionsAH_AHbrowseProfitText:SetText(L["Show Profit in Auctions"])
	AuctionProfit_InterfaceOptionsAH_AHbrowseProfit:SetChecked(AuctionProfit_Options.AHBrowseTabProfit)
	AuctionProfit_InterfaceOptionsAH_AHbrowseProfit:SetScript("OnClick", function(self)
		AuctionProfit_Options.AHBrowseTabProfit = not AuctionProfit_Options.AHBrowseTabProfit
		if not AuctionProfit_Options.AHBrowseTabProfit then
			if APFrame_AHBrowseTabFrame1 then
				for i=1, NUM_BROWSE_TO_DISPLAY do
					_G["APFrame_AHBrowseTabFrame"..i]:Hide()
				end
			end
		end
	end)
	AHbrowseProfit.tooltipText = L["Enable or disable the display of your profit under the name of the seller in Auction House Tab 'Auction Browse'.\n\nIf you are at the Auction House while changing this option: This is active after the next auction update (new search / scroll up or down)."]

	local AHTooltip = CreateFrame("CheckButton", "AuctionProfit_InterfaceOptionsAH_AHTooltip", AuctionProfit_InterfaceOptionsAH, "InterfaceOptionsCheckButtonTemplate")
	AuctionProfit_InterfaceOptionsAH_AHTooltip:SetPoint("TOPLEFT", AHbrowseProfit, "TOPLEFT", 0, -20)
	AuctionProfit_InterfaceOptionsAH_AHTooltipText:SetText(L["Enable/Disable Tooltip"])
	AuctionProfit_InterfaceOptionsAH_AHTooltip:SetChecked(AuctionProfit_Options.AHTooltipToggle)
	AHTooltip.tooltipText = L["Enable or disable AuctionProfit data in Auction House tooltip."]

	local AHTTlines = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsAH_AHTTlines", AuctionProfit_InterfaceOptionsAH, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetWidth(200)
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetHeight(16)
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetPoint("TOPLEFT", AHTooltip, "TOPLEFT", 28, -40)
	AuctionProfit_InterfaceOptionsAH_AHTTlinesText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsAH_AHTTlinesText:SetText(L["Tooltip Lines"]..": "..AuctionProfit_Options.AHTooltipLines)
	AuctionProfit_InterfaceOptionsAH_AHTTlinesLow:SetText("0")
	AuctionProfit_InterfaceOptionsAH_AHTTlinesHigh:SetText("40")
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetMinMaxValues(0, 40)
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetValueStep(1)
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetValue(AuctionProfit_Options.AHTooltipLines)
	AuctionProfit_InterfaceOptionsAH_AHTTlines:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsAH_AHTTlinesText:SetText(L["Tooltip Lines"]..": "..value)
		AuctionProfit_Options.AHTooltipLines = value
	end)
	AHTTlines.tooltipText = L["Maximum Lines in Auction House Tooltip.\n0 = Only Profit"]

	local AHTTposition = CreateFrame("Frame", "AuctionProfit_InterfaceOptionsAH_AHTTposition", AuctionProfit_InterfaceOptionsAH, "UIDropDownMenuTemplate")
	AuctionProfit_InterfaceOptionsAH_AHTTposition:SetPoint("TOPLEFT", AHTTlines, "TOPLEFT", -16, -50)
	AuctionProfit_InterfaceOptionsAH_AHTTposition:CreateFontString("AuctionProfit_InterfaceOptionsAH_AHTTpositionLabel", "BACKGROUND", "GameFontHighlight")
	AuctionProfit_InterfaceOptionsAH_AHTTpositionLabel:SetPoint("BOTTOMLEFT", AuctionProfit_InterfaceOptionsAH_AHTTposition, "TOPLEFT", 16, 5)
	AuctionProfit_InterfaceOptionsAH_AHTTpositionLabel:SetText(L["Tooltip Position"]..":")
	AuctionProfit_InterfaceOptionsAH_AHTTposition.initialize = function()
		for k,v in pairs(ttpos) do
			info.text = k
			info.arg1 = k
			info.arg2 = nil
			info.func = function(button, selected)
				AuctionProfit_Options.AHTooltipPosAnchor = selected
				AuctionProfit_InterfaceOptionsAH_AHTTpositionText:SetText(AuctionProfit_Options.AHTooltipPosAnchor)
				UIDropDownMenu_SetWidth(AuctionProfit_InterfaceOptionsAH_AHTTposition, AuctionProfit_InterfaceOptionsAH_AHTTpositionText:GetWidth()+25)
			end
			if AuctionProfit_Options.AHTooltipPosAnchor == info.arg1 then
				info.checked = 1
			else
				info.checked = nil
			end
			info.keepShownOnClick = nil
			info.tooltipTitle = k
			info.tooltipText = v
			UIDropDownMenu_AddButton(info)
		end
	end
	AuctionProfit_InterfaceOptionsAH_AHTTpositionText:SetText(AuctionProfit_Options.AHTooltipPosAnchor)
	UIDropDownMenu_SetWidth(AuctionProfit_InterfaceOptionsAH_AHTTposition, AuctionProfit_InterfaceOptionsAH_AHTTpositionText:GetWidth()+25)

	local AHTTposX = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsAH_AHTTposX", AuctionProfit_InterfaceOptionsAH, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetWidth(200)
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetHeight(16)
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetPoint("TOPLEFT", AHTTposition, "TOPLEFT", 16, -50)
	AuctionProfit_InterfaceOptionsAH_AHTTposXText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsAH_AHTTposXText:SetText(L["Pos X"]..": "..AuctionProfit_Options.AHTooltipPosX)
	AuctionProfit_InterfaceOptionsAH_AHTTposXLow:SetText("-600")
	AuctionProfit_InterfaceOptionsAH_AHTTposXHigh:SetText("600")
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetMinMaxValues(-600, 600)
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetValueStep(10)
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetValue(AuctionProfit_Options.AHTooltipPosX)
	AuctionProfit_InterfaceOptionsAH_AHTTposX:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsAH_AHTTposXText:SetText(L["Pos X"]..": "..value)
		AuctionProfit_Options.AHTooltipPosX = value
	end)
	AHTTposX.tooltipText = L["Pos X"]

	local AHTTposY = CreateFrame("Slider", "AuctionProfit_InterfaceOptionsAH_AHTTposY", AuctionProfit_InterfaceOptionsAH, "OptionsSliderTemplate")
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetWidth(200)
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetHeight(16)
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetPoint("TOPLEFT", AHTTposX, "TOPLEFT", 0, -50)
	AuctionProfit_InterfaceOptionsAH_AHTTposYText:SetPoint("LEFT")
	AuctionProfit_InterfaceOptionsAH_AHTTposYText:SetText(L["Pos X"]..": "..AuctionProfit_Options.AHTooltipPosY)
	AuctionProfit_InterfaceOptionsAH_AHTTposYLow:SetText("-50")
	AuctionProfit_InterfaceOptionsAH_AHTTposYHigh:SetText("50")
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetMinMaxValues(-50, 50)
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetValueStep(5)
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetValue(AuctionProfit_Options.AHTooltipPosY)
	AuctionProfit_InterfaceOptionsAH_AHTTposY:SetScript("OnValueChanged", function(self, value)
		AuctionProfit_InterfaceOptionsAH_AHTTposYText:SetText(L["Pos Y"]..": "..value)
		AuctionProfit_Options.AHTooltipPosY = value
	end)
	AHTTposY.tooltipText = L["Pos Y"]

	if AuctionProfit_Options.AHTooltipToggle then
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAH_AHTTlines)
		UIDropDownMenu_EnableDropDown(AuctionProfit_InterfaceOptionsAH_AHTTposition)
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAH_AHTTposX)
		BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAH_AHTTposY)
	else
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAH_AHTTlines)
		UIDropDownMenu_DisableDropDown(AuctionProfit_InterfaceOptionsAH_AHTTposition)
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAH_AHTTposX)
		BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAH_AHTTposY)
	end

	AuctionProfit_InterfaceOptionsAH_AHTooltip:SetScript("OnClick", function(self)
		AuctionProfit_Options.AHTooltipToggle = not AuctionProfit_Options.AHTooltipToggle
		if AuctionProfit_Options.AHTooltipToggle then
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAH_AHTTlines)
			UIDropDownMenu_EnableDropDown(AuctionProfit_InterfaceOptionsAH_AHTTposition)
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAH_AHTTposX)
			BlizzardOptionsPanel_Slider_Enable(AuctionProfit_InterfaceOptionsAH_AHTTposY)
		else
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAH_AHTTlines)
			UIDropDownMenu_DisableDropDown(AuctionProfit_InterfaceOptionsAH_AHTTposition)
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAH_AHTTposX)
			BlizzardOptionsPanel_Slider_Disable(AuctionProfit_InterfaceOptionsAH_AHTTposY)
		end
	end)
	-- panel Auction House END

	InterfaceOptions_AddCategory(options)
	InterfaceOptions_AddCategory(optionsPlayer)
	InterfaceOptions_AddCategory(optionsAPframe)
	InterfaceOptions_AddCategory(optionsAH)
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:DataUpdate()
	if APserver and #APserver > 0 then
		for i=1, #APserver do
			if APserver[i] == AP_ServerName then
				AP_ServerNameIndex = i
				break
			end
		end
	end
	local un = 1
	if AP_ServerNameIndex then
		AP_NameCount = #APbuyer[AP_ServerNameIndex]
		for i=1, AP_NameCount do
			local data = APbuyer[AP_ServerNameIndex][i]
			local pos = string_find(data, ":", 1)
			local str = string_sub(data, 1, pos-1)
			-- If you recieved an invoice auction house mail from a player that was delete from server the
			-- AuctionProfit TakeInboxMoney function saved an emtpy string to the db (my bad) :(
			-- This was fixed in r68158.
			-- This check is to prevent such errors by translating such "" string(s) to "UnknownName" and save it.
			if str == "" then
				str = "UnknownName"..un
				APbuyer[AP_ServerNameIndex][i] = str..data
				un = un + 1
			end
			AP_NameIndex[i] = str
		end
		AuctionProfit:DataCheck()
		AuctionProfit:StatusLineTxt()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:DataCheck()
	if StartCheck then return end
	local x = 1
	for i=1, #APitemName do
		if itemNameServerCheckReturn(APitemName[i],AP_ServerNameIndex) then
			AP_ItemIndex[x] = i
			AP_ItemNames[x] = itemNameReturn(APitemName[i])
			x = x + 1
		end
	end
	AP_ItemCount = #AP_ItemIndex
	StartCheck = 1
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:SaveData(invoiceType, itemName, playerName, bid, buyout, deposit, consignment)
--local x1 = GetTime() --TIMEcheck

		if not playerName then return end
		if type(playerName) ~= "string" then return end
		if playerName == "" then return end
		if not itemName then return end
		if type(itemName) ~= "string" then return end
		if itemName == "" then return end
		
		local AP_profit
		if bid == buyout then AP_profit = buyout - consignment end -- buyout
		if bid < buyout  then AP_profit = bid - consignment end    -- bid
		if buyout == 0   then AP_profit = bid - consignment end    -- bid (no buyout)
		
		if not AP_profit then return end

		-- -----------------------------------------------------------------------------------------------------------------
		-- APserver save / update START ----------
		local APserverNum = #APserver
		local AP_ServerNameIDX = nil
		for i=1, APserverNum do
			if APserver[i] == AP_ServerName then
				AP_ServerNameIDX = i
				break
			end
		end
		if AP_ServerNameIDX == nil and APserverNum > 0 then
			local newServerNum = APserverNum+1
			APserver[newServerNum] = AP_ServerName
			AP_ServerNameIDX = newServerNum
		end
		if AP_ServerNameIDX == nil then
			APserver[1] = AP_ServerName
			AP_ServerNameIDX = 1
		end
		-- APserver save / update END ----------
		-- -----------------------------------------------------------------------------------------------------------------

		-- -----------------------------------------------------------------------------------------------------------------
		-- APitemName new entry / update START ----------
		local APitemNameNum = #APitemName
		local AP_itemNameIDX = nil
		-- APitemName update with entries and with itemName START ----------
		for i=1, APitemNameNum do
			if itemNameReturn(APitemName[i]) == itemName then
				AP_itemNameIDX = i
				-- check if item has server entry (AP_ServerNameIDX)
				-- yes(1) = update | no(else 0) = add new entry
				if itemNameServerCheckReturn(APitemName[i],AP_ServerNameIDX) then
					local AP_itemNameIDX_profit = tonumber( itemNameSubDataPartProfitReturn(APitemName[i],AP_ServerNameIDX) )
					local AP_itemNameIDX_buys = tonumber( itemNameSubDataPartBuysReturn(APitemName[i],AP_ServerNameIDX) )
					AP_itemNameIDX_profit = AP_itemNameIDX_profit + AP_profit
					AP_itemNameIDX_buys = AP_itemNameIDX_buys + 1
					local AP_itemNameIDX_subData = tostring(AP_ServerNameIDX).."-"..tostring(AP_itemNameIDX_profit).."-"..tostring(AP_itemNameIDX_buys)
					local AP_itemNameIDX_subDataOLD = itemNameSubDataPartReturn(APitemName[i],AP_ServerNameIDX)
					-- replace ... search for subDataOLD and replace with subData
					local AP_itemNameIDX_subData_replace = itemNameSubDataPartReplaceReturn(APitemName[i],AP_itemNameIDX_subDataOLD,AP_itemNameIDX_subData)
					APitemName[i] = itemName.."#"..AP_itemNameIDX_subData_replace
				else
					APitemName[i] = APitemName[i]..":"..tostring(AP_ServerNameIDX).."-"..tostring(AP_profit).."-1"
					-- update AP_ItemIndex
					AP_ItemCount = AP_ItemCount + 1
					AP_ItemIndex[AP_ItemCount] = AP_itemNameIDX
					AP_ItemNames[AP_ItemCount] = itemName
					-- update AP_ItemIndex
				end
				break
			end
		end
		-- APitemName update with entries and with itemName END ----------
		-- APitemName update with entries but not with itemName START ----------
		if AP_itemNameIDX == nil and APitemNameNum > 0 then
			APitemName[APitemNameNum+1] = itemName.."#"..tostring(AP_ServerNameIDX).."-"..tostring(AP_profit).."-1"
			AP_itemNameIDX = APitemNameNum+1
			-- update AP_ItemIndex
			AP_ItemCount = AP_ItemCount + 1
			AP_ItemIndex[AP_ItemCount] = AP_itemNameIDX
			AP_ItemNames[AP_ItemCount] = itemName
			-- update AP_ItemIndex
		end
		-- APitemName update with entries but not with itemName END ----------
		-- APitemName update with no entries START
		if AP_itemNameIDX == nil then
			APitemName[1] = itemName.."#"..tostring(AP_ServerNameIDX).."-"..tostring(AP_profit).."-1"
			AP_itemNameIDX = 1
			-- update AP_ItemIndex
			AP_ItemCount = 1
			AP_ItemIndex = {}
			AP_ItemIndex[AP_ItemCount] = AP_itemNameIDX
			AP_ItemNames[AP_ItemCount] = itemName
			-- update AP_ItemIndex
		end
		-- APitemName update with no entries END ----------
		-- APitemName new entry / update END ----------
		-- -----------------------------------------------------------------------------------------------------------------

		-- -----------------------------------------------------------------------------------------------------------------
		-- APbuyer new entry / update START ----------
		if APbuyer[AP_ServerNameIDX] == nil
			then APbuyer[AP_ServerNameIDX] = {}
		end
		local APbuyerNum = #APbuyer[AP_ServerNameIDX]
		local AP_buyerNameIDX = nil
		-- APbuyer update with entries and with playerName START ----------
		for i=1, APbuyerNum do
			--local AP_buyer = APbuyer[AP_ServerNameIDX][i]
			--local AP_buyerName = buyerNameReturn(AP_buyer)
			if AP_NameIndex[i] == playerName then -- AP_NameIndex keys are equal to APbuyer[AP_ServerNameIDX]
			--if AP_buyerName == playerName then
				AP_buyerNameIDX = i
				local AP_buyer = APbuyer[AP_ServerNameIDX][i]
				
				-- update APbuyer MainData
				local AP_buyerNameIDX_maindata = buyerMainDataReturn(AP_buyer)
				local AP_buyerNameIDX_profit = tonumber( strReturn(AP_buyerNameIDX_maindata ,2) )
				local AP_buyerNameIDX_buys = tonumber( strReturn(AP_buyerNameIDX_maindata ,3) )
				AP_buyerNameIDX_profit = AP_buyerNameIDX_profit + AP_profit
				AP_buyerNameIDX_buys = AP_buyerNameIDX_buys + 1
				AP_buyerNameIDX_mainData = playerName..":"..tostring(AP_buyerNameIDX_profit)..":"..tostring(AP_buyerNameIDX_buys)
				-- update APbuyer MainData

				-- update APbuyer SubData
				-- check if buyer has itemName: yes(1) = update | no(else 0) = add new entry
				if buyerSubDataItemIDcheckReturn(AP_buyer,AP_itemNameIDX) then
					local AP_buyerNameIDX_subData_profit = AP_profit + tonumber(buyerSubDataPartProfitReturn(AP_buyer,AP_itemNameIDX))
					local AP_buyerNameIDX_subData_buys = tonumber(buyerSubDataPartBuysReturn(AP_buyer,AP_itemNameIDX)) + 1
					local AP_buyerNameIDX_subData = tostring(AP_itemNameIDX).."-"..tostring(AP_buyerNameIDX_subData_profit).."-"..tostring(AP_buyerNameIDX_subData_buys)
					local AP_buyerNameIDX_subDataOLD = buyerSubDataPartReturn(AP_buyer,AP_itemNameIDX)
					-- replace ... search for subDataOLD and replace with subData
					local AP_buyerNameIDX_subData_replace = buyerSubDataPartReplaceReturn(AP_buyer,AP_buyerNameIDX_subDataOLD,AP_buyerNameIDX_subData)
					APbuyer[AP_ServerNameIDX][i] = AP_buyerNameIDX_mainData.."#"..AP_buyerNameIDX_subData_replace
				else
					local AP_buyerNameIDX_subData = tostring(AP_itemNameIDX).."-"..tostring(AP_profit).."-1"
					local AP_buyerNameIDX_subDataOLD = buyerSubDataReturn(AP_buyer)
					APbuyer[AP_ServerNameIDX][i] = AP_buyerNameIDX_mainData.."#"..AP_buyerNameIDX_subDataOLD..":"..AP_buyerNameIDX_subData
				end
				-- update APbuyer SubData
				break
			end
		end
		-- APbuyer update with entries and with playerName END ----------
		-- APbuyer update with entries but not with playerName START ----------
		if AP_buyerNameIDX == nil and APbuyerNum > 0 then
			APbuyer[AP_ServerNameIDX][APbuyerNum+1] = playerName..":"..tostring(AP_profit)..":1#"..tostring(AP_itemNameIDX).."-"..tostring(AP_profit).."-1"
			AP_buyerNameIDX = APbuyerNum+1
		end
		-- APbuyer update with entries but not with playerName END ----------
		-- APbuyer update with no entries START ----------
		if AP_buyerNameIDX == nil then
			APbuyer[AP_ServerNameIDX][1] = playerName..":"..tostring(AP_profit)..":1#"..tostring(AP_itemNameIDX).."-"..tostring(AP_profit).."-1"
			AP_buyerNameIDX = 1
		end
		-- APbuyer update with no entries END ----------
		-- APbuyer new entry / update START ----------
		-- -----------------------------------------------------------------------------------------------------------------

		-- -----------------------------------------------------------------------------------------------------------------
		-- APlastAuctions save / update START ----------
		if APlastAuctions[AP_ServerNameIDX] == nil then
			APlastAuctions[AP_ServerNameIDX] = {}
		end
		local APlastAuctionsNum = #APlastAuctions[AP_ServerNameIDX]
		local AP_lastAuctionsIDX = nil
		if APlastAuctionsNum == AP_LastXAuctions then
			for i=1, (APlastAuctionsNum-1) do
				APlastAuctions[AP_ServerNameIDX][i] = APlastAuctions[AP_ServerNameIDX][i+1]
			end
			AP_lastAuctionsIDX = AP_LastXAuctions
		else
			AP_lastAuctionsIDX = APlastAuctionsNum+1
		end
		if AP_lastAuctionsIDX == nil then
			AP_lastAuctionsIDX = 1
		end
		local xdate = date("%y%m%d %H%M")
		APlastAuctions[AP_ServerNameIDX][AP_lastAuctionsIDX] = xdate.."#"..tostring(AP_itemNameIDX)..":"..tostring(AP_profit)..":"..tostring(AP_buyerNameIDX)
		-- APlastAuctions save / update END ----------
		-- -----------------------------------------------------------------------------------------------------------------

		if AuctionProfit_Options.ChatOutput then
			Print("|cff"..colorGreen..itemName.."|r | |cff"..colorBlue..L["Buyer"]..":|r |cff"..colorYellow..playerName.."|r | |cff"..colorBlue..L["Profit"]..":|r "..moneyFormat(AP_profit))
		end
		AuctionProfit:DataUpdate()

		if AuctionProfit_Options.APwindowToggle then
			AuctionProfit:DisplayMode(2)
			APFrame:Show()
		end
--local x3 = GetTime() Print("time save: "..x3-x1) --TIMEcheck
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:OnVerticalScroll(self, value)
	APFrame_ScrollFrameScrollBar:SetValue(value)
	APFrame_ScrollFrame.offset = math_floor((value / 12) + 0.5)
	AuctionProfit:FillList()
end

function AuctionProfit:FillList(sort, order, data)
	if not AP_ServerNameIndex then return end

	local AP_showtableNumSize = 20

	if     data == 0 then dbCount = AP_NameCount
	elseif data == 1 then dbCount = AP_ItemCount
	elseif data == 2 then dbCount = #APlastAuctions[AP_ServerNameIndex] end

 	if sort and order and data then
		AuctionProfit:SortTable(sort, order, data)
	end

	if #AP_ListData ~= dbCount then
		dbCount = #AP_ListData
	end

	if dbCount and dbCount < AP_showtableNumSize then
		for i=1, AP_showtableNumSize do
			_G["APList"..i]:Hide()
		end
		AP_showtableNumSize = dbCount
	end

	FauxScrollFrame_Update(APFrame_ScrollFrame, dbCount, AP_showtableNumSize, 12)

	if dbCount <= 20 then
		APFrame_ScrollFrame:Hide()
	else
		APFrame_ScrollFrame:Show()
	end

	local j = 1
	local sliderPos = APFrame_ScrollFrame.offset -- FauxScrollFrame_GetOffset(APFrame_ScrollFrame)
	for i=sliderPos+1, sliderPos+AP_showtableNumSize do
		if sliderPos < dbCount then
			local part1, part2, part3 = strReturn(AP_ListData[i], 0)
			_G["APList"..j.."Name"]:SetText( part1 )
			_G["APList"..j.."Buys"]:SetText( part3 )
			_G["APList"..j.."Profit"]:SetText( moneyFormat( tonumber(part2) ) )
			_G["APList"..j]:Show()
			j = j + 1
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:SortTable(sort, order, data)
	--Print("sort="..sort.." | order="..order.." | data="..data) --BUGcheck
	--local x1 = GetTime() --TIMEcheck
	-- arrows
	for i=1, 3 do
		local ArrowG
		if     i == 1 then ArrowG = _G["APBrowseSortNameArrowArrow"]       --default sort = 0
		elseif i == 2 then ArrowG = _G["APBrowseSortBuysArrowArrow"]       --default sort = 1
		elseif i == 3 then ArrowG = _G["APBrowseSortProfitArrowArrow"] end --default sort = 1
		if (sort+1) == i then
			if (i==2 or i==3) then
				if     order == 1 then ArrowG:SetTexCoord(0.5625, 0, 1, 0)
				elseif order == 0 then ArrowG:SetTexCoord(0, 0.5625, 0, 1) end
			else
				if     order == 1 then ArrowG:SetTexCoord(0.5625, 0, 1, 0)
				elseif order == 0 then ArrowG:SetTexCoord(0, 0.5625, 0, 1) end
			end
		else
			ArrowG:SetTexCoord(0, 0, 0, 0)
		end		
		if data == 2 then
			ArrowG:SetTexCoord(0, 0, 0, 0)
		end
	end
	-- arrows

	-- -------------------------------------------------------------------------------------------------------------------
	-- sort by name START --------------------------------------------------------
	if sort == 0 then
		AP_sortType = 0
		if     data == 0 then dbCount = AP_NameCount
		elseif data == 1 then dbCount = AP_ItemCount
		elseif data == 2 then dbCount = #APlastAuctions[AP_ServerNameIndex] end

		AP_ListData = {}
		local ii = 1
		-- -------------------------------------------------------------------------
		if data == 0 then
			for i = 1, dbCount do
				if SearchQuery then
					local str = buyerMainDataReturn(APbuyer[AP_ServerNameIndex][i])
					if string_find(string_lower(str),string_lower(SearchQuery),1,true) then
						AP_ListData[ii] = str
						ii = ii + 1
					end
				else
					AP_ListData[i] = buyerMainDataReturn(APbuyer[AP_ServerNameIndex][i])
				end
			end
			AP_sortData = 0
		-- -------------------------------------------------------------------------
		elseif data == 1 then
			for i = 1, dbCount do
				local ItemData = APitemName[AP_ItemIndex[i]]
				if SearchQuery then
					if string_find(string_lower(ItemData),string_lower(SearchQuery),1,true) then
						AP_ListData[ii] = itemNameReturn(ItemData)..":"..itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex)..":"..itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex)
						ii = ii + 1
					end
				else
					AP_ListData[i] = itemNameReturn(ItemData)..":"..itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex)..":"..itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex)
				end
			end
			AP_sortData = 1
		-- -------------------------------------------------------------------------
		elseif data == 2 then
			for i = 1, dbCount do
				AP_ListData[(dbCount-i+1)] = lastAuctionsReturn(APlastAuctions[AP_ServerNameIndex][i],AP_ServerNameIndex)
			end
			AP_sortData = 2
		end
		-- -------------------------------------------------------------------------

		if data == 2 then return end
		
		if order == 0 then
			table_sort(AP_ListData, function(a,b) return a<b end)
			AP_sortOrder = 0
		elseif order == 1 then
			table_sort(AP_ListData, function(a,b) return a>b end)
			AP_sortOrder = 1
		end
	-- sort by name END ----------------------------------------------------------
	-- -------------------------------------------------------------------------------------------------------------------

	-- -------------------------------------------------------------------------------------------------------------------
	-- sort by buys START --------------------------------------------------------
	elseif sort == 1 then
		AP_sortType = 1
		if     data == 0 then dbCount = AP_NameCount
		elseif data == 1 then dbCount = AP_ItemCount
		elseif data == 2 then dbCount = #APlastAuctions[AP_ServerNameIndex] end

		AP_ListData = {}
		local AP_ListData1 = {}
		local AP_ListData2 = {}
		-- -------------------------------------------------------------------------
		if data == 0 then
			for i = 1, dbCount do
				local MainData = buyerMainDataReturn(APbuyer[AP_ServerNameIndex][i])
				AP_ListData1[i] = tonumber(strReturn(MainData,3)) -- buys
				AP_ListData2[i] = tonumber(strReturn(MainData,2)) -- profit
			end
			AP_sortData = 0
		-- -------------------------------------------------------------------------
		elseif data == 1 then
			for i = 1, dbCount do
				local ItemData = APitemName[AP_ItemIndex[i]]
				AP_ListData1[i] = tonumber(itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex))   -- buys
				AP_ListData2[i] = tonumber(itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex)) -- profit
			end
			AP_sortData = 1
		-- -------------------------------------------------------------------------
		elseif data == 2 then
			for i = 1, dbCount do
				AP_ListData[(dbCount-i+1)] = lastAuctionsReturn(APlastAuctions[AP_ServerNameIndex][i],AP_ServerNameIndex)
			end
			AP_sortData = 2
		end
		-- -------------------------------------------------------------------------

		if data == 2 then return end

		table_sort(AP_ListData1, function(a,b) return a>b end)
		table_sort(AP_ListData2, function(a,b) return a>b end)

		local AP_ListData1_xl = string_len(tostring(AP_ListData1[1]))
		local AP_ListData2_xl = string_len(tostring(AP_ListData2[1]))

		local ii = 1
		-- -------------------------------------------------------------------------
		if data == 0 then
			for i = 1, dbCount do
				local Buyer = APbuyer[AP_ServerNameIndex][i]
				local BuyerMainData = buyerMainDataReturn(Buyer)
				if SearchQuery then
					local str = numMatrix(strReturn(BuyerMainData,3),AP_ListData1_xl)..numMatrix(strReturn(BuyerMainData,2),AP_ListData2_xl)..Buyer
					local chk = strReturn(BuyerMainData,3)..strReturn(BuyerMainData,2)..Buyer
					if string_find(string_lower(chk),string_lower(SearchQuery),1,true) then
						AP_ListData[ii] = str
						ii = ii + 1
					end
				else
					AP_ListData[i] = numMatrix(strReturn(BuyerMainData,3),AP_ListData1_xl)..numMatrix(strReturn(BuyerMainData,2),AP_ListData2_xl)..Buyer
				end
			end
		-- -------------------------------------------------------------------------
		elseif data == 1 then
			for i = 1, dbCount do
				local item = AP_ItemIndex[i]
				local ItemData = APitemName[item]
				if SearchQuery then
					local str = numMatrix(itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex),AP_ListData1_xl)..numMatrix(itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex),AP_ListData2_xl)..ItemData
					local chk = itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex)..itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex)..ItemData
					if string_find(string_lower(chk),string_lower(SearchQuery),1,true) then
						AP_ListData[ii] = str
						ii = ii + 1
					end
				else
					AP_ListData[i] = numMatrix(itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex),AP_ListData1_xl)..numMatrix(itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex),AP_ListData2_xl)..ItemData
				end
			end
		end
		-- -------------------------------------------------------------------------
		
		if order == 0 then
			table_sort(AP_ListData, function(a,b) return a>b end)
			AP_sortOrder = 0
		elseif order == 1 then
			table_sort(AP_ListData, function(a,b) return a<b end)
			AP_sortOrder = 1
		end

		if data == 0 then
			for i = 1, #AP_ListData do
				AP_ListData[i] = buyerMainDataReturn(string_sub(AP_ListData[i],(AP_ListData1_xl+AP_ListData2_xl+1)))
			end
		elseif data == 1 then
			for i = 1, #AP_ListData do
				AP_ListData[i] = string_sub(itemNameReturn(AP_ListData[i])..":"..itemNameSubDataPartProfitReturn(AP_ListData[i],AP_ServerNameIndex)..":"..itemNameSubDataPartBuysReturn(AP_ListData[i],AP_ServerNameIndex),(AP_ListData1_xl+AP_ListData2_xl+1))
			end
		end

		table_wipe(AP_ListData1)
		table_wipe(AP_ListData2)
	-- sort by buys END ----------------------------------------------------------
	-- -------------------------------------------------------------------------------------------------------------------

	-- -------------------------------------------------------------------------------------------------------------------
	-- sort by profit START ------------------------------------------------------
	elseif sort == 2 then
		AP_sortType = 2
		if     data == 0 then dbCount = AP_NameCount
		elseif data == 1 then dbCount = AP_ItemCount
		elseif data == 2 then dbCount = #APlastAuctions[AP_ServerNameIndex] end

		AP_ListData = {}
		local AP_ListData1 = {}
		local AP_ListData2 = {}
		-- -------------------------------------------------------------------------
		if data == 0 then
			for i = 1, dbCount do
				local MainData = buyerMainDataReturn(APbuyer[AP_ServerNameIndex][i])
				AP_ListData1[i] = tonumber(strReturn(MainData,3)) -- buys
				AP_ListData2[i] = tonumber(strReturn(MainData,2)) -- profit
			end
			AP_sortData = 0
		-- -------------------------------------------------------------------------
		elseif data == 1 then
			for i = 1, dbCount do
				local ItemData = APitemName[AP_ItemIndex[i]]
				AP_ListData1[i] = tonumber(itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex))   -- buys
				AP_ListData2[i] = tonumber(itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex)) -- profit
			end
			AP_sortData = 1
		-- -------------------------------------------------------------------------
		elseif data == 2 then
			for i = 1, dbCount do
				AP_ListData[(dbCount-i+1)] = lastAuctionsReturn(APlastAuctions[AP_ServerNameIndex][i],AP_ServerNameIndex)
			end
			AP_sortData = 2
		end
		-- -------------------------------------------------------------------------

		if data == 2 then return end

		table_sort(AP_ListData1, function(a,b) return a>b end)
		table_sort(AP_ListData2, function(a,b) return a>b end)

		local AP_ListData1_xl = string_len(tostring(AP_ListData1[1]))
		local AP_ListData2_xl = string_len(tostring(AP_ListData2[1]))

		local ii = 1
		-- -------------------------------------------------------------------------
		if data == 0 then
			for i = 1, dbCount do
				local Buyer = APbuyer[AP_ServerNameIndex][i]
				local BuyerMainData = buyerMainDataReturn(Buyer)
				if SearchQuery then
					local str = numMatrix(strReturn(BuyerMainData,2),AP_ListData2_xl)..numMatrix(strReturn(BuyerMainData,3),AP_ListData1_xl)..Buyer
					local chk = strReturn(BuyerMainData,2)..strReturn(BuyerMainData,3)..Buyer
					if string_find(string_lower(chk),string_lower(SearchQuery),1,true) then
						AP_ListData[ii] = str
						ii = ii + 1
					end
				else
					AP_ListData[i] = numMatrix(strReturn(BuyerMainData,2),AP_ListData2_xl)..numMatrix(strReturn(BuyerMainData,3),AP_ListData1_xl)..Buyer
				end
			end
		-- -------------------------------------------------------------------------
		elseif data == 1 then
			for i = 1, dbCount do
				local item = AP_ItemIndex[i]
				local ItemData = APitemName[item]
				if SearchQuery then
					local str = numMatrix(itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex),AP_ListData2_xl)..numMatrix(itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex),AP_ListData1_xl)..ItemData
					local chk = itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex)..itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex)..ItemData
					if string_find(string_lower(chk),string_lower(SearchQuery),1,true) then
						AP_ListData[ii] = str
						ii = ii + 1
					end
				else
					AP_ListData[i] = numMatrix(itemNameSubDataPartProfitReturn(ItemData,AP_ServerNameIndex),AP_ListData2_xl)..numMatrix(itemNameSubDataPartBuysReturn(ItemData,AP_ServerNameIndex),AP_ListData1_xl)..ItemData
				end
			end
		end
		-- -------------------------------------------------------------------------

		if order == 0 then
			table_sort(AP_ListData, function(a,b) return a>b end)
			AP_sortOrder = 0
		elseif order == 1 then
			table_sort(AP_ListData, function(a,b) return a<b end)
			AP_sortOrder = 1
		end

		if data == 0 then
			for i = 1, #AP_ListData do
				AP_ListData[i] = buyerMainDataReturn(string_sub(AP_ListData[i],(AP_ListData1_xl+AP_ListData2_xl+1)))
			end
		elseif data == 1 then
			for i = 1, #AP_ListData do
				AP_ListData[i] = string_sub(itemNameReturn(AP_ListData[i])..":"..itemNameSubDataPartProfitReturn(AP_ListData[i],AP_ServerNameIndex)..":"..itemNameSubDataPartBuysReturn(AP_ListData[i],AP_ServerNameIndex),(AP_ListData1_xl+AP_ListData2_xl+1))
			end
		end
		table_wipe(AP_ListData1)
		table_wipe(AP_ListData2)
	end
	-- sort by profit END --------------------------------------------------------
	-- -------------------------------------------------------------------------------------------------------------------
	--local x2 = GetTime() Print("Sort Table: "..x2-x1) --TIMEcheck
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:InitFrame()
	-- localize buttons and set width
	APFrame_headline_ButtonName:SetText(L["Buyer"])
	APFrame_headline_ButtonName:SetWidth( tonumber(APFrame_headline_ButtonName:GetTextWidth())+20 )

	APFrame_headline_ButtonItem:SetText(L["Items"])
	APFrame_headline_ButtonItem:SetWidth( tonumber(APFrame_headline_ButtonItem:GetTextWidth())+20 )

	APFrame_headline_ButtonLastAuctions:SetText(string_format(L["Last %s Auctions"],AP_LastXAuctions))
	APFrame_headline_ButtonLastAuctions:SetWidth( tonumber(APFrame_headline_ButtonLastAuctions:GetTextWidth())+20 )
	
	APFrame_headline_ButtonOptions:SetText(L["Options"])
	APFrame_headline_ButtonOptions:SetWidth( tonumber(APFrame_headline_ButtonOptions:GetTextWidth())+20 )

	APFrame_searchBox_ClearButton:SetText(L["Clear"])
	APFrame_searchBox_ClearButton:SetWidth( tonumber(APFrame_searchBox_ClearButton:GetTextWidth())+20 )

	APFrame_searchBox_SearchText:SetText(L["search:"])

	-- scroll knob tweak
	APFrame_ScrollFrameScrollBarThumbTexture:SetWidth(16)
	APFrame_ScrollFrameScrollBarThumbTexture:SetHeight(16)
	APFrame_ScrollFrameScrollBarThumbTexture:SetTexCoord(0.25, 0.75, 0.25, 0.75)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:UpdateFrame(num, start)
	if AP_ServerNameIndex then
		APFrame_headline_ButtonName:Enable()
		APFrame_headline_ButtonItem:Enable()
		APFrame_headline_ButtonLastAuctions:Enable()
		APFrame_headline_ButtonSearch:Enable()
		APFrame_ListView:Show()
		APFrame_sort:Show()
		APFrame_StatusLine:ClearAllPoints()
		APFrame_StatusLine:SetPoint("TOPLEFT",APFrame_ListView,"BOTTOMLEFT",0,0)
		if AuctionProfit_Options.APwindowSearchShow then
			AuctionProfit:SearchFrameShow()
		else
			AuctionProfit:SearchFrameHide()
		end
	else
		APFrame_headline_ButtonName:Disable()
		APFrame_headline_ButtonItem:Disable()
		APFrame_headline_ButtonLastAuctions:Disable()
		APFrame_headline_ButtonSearch:Disable()
		APFrame_ListView:Hide()
		APFrame_sort:Hide()
		APFrame_StatusLine:ClearAllPoints()
		APFrame_StatusLine:SetPoint("TOPLEFT",APFrame_headline,"BOTTOMLEFT",0,0)
		APFrame_StatusLineTxt:SetText(L["No Data!"])
		return
	end

	if num == 0 then
		local col1 = 90
		local col1width = 110
		local col2 = 200
		local col2width = 90
		local col3 = 290
		local col3width = 100
		setHeadlineTabButton("APFrame_headline_ButtonName", true)
		setHeadlineTabButton("APFrame_headline_ButtonItem", nil)
		setHeadlineTabButton("APFrame_headline_ButtonLastAuctions", nil)

		APFrame_searchAlignFrame:Show()

		APBrowseSortName:Enable()
		APBrowseSortBuys:Enable()
		APBrowseSortProfit:Enable()
		APBrowseSortNameText:SetText(L["Name"])
		APBrowseSortNameText:ClearAllPoints()
		APBrowseSortNameText:SetPoint("LEFT",0,0)
		APBrowseSortNameArrow:ClearAllPoints()
		APBrowseSortNameArrow:SetPoint("LEFT","APBrowseSortNameText","RIGHT",3,0)
		APBrowseSortName:SetWidth(col1width)
		APBrowseSortName:SetPoint("LEFT","APFrame_sort","LEFT",col1,0)
		APBrowseSortBuysText:SetText(L["Auctions"])
		APBrowseSortBuysText:ClearAllPoints()
		APBrowseSortBuysText:SetPoint("CENTER",0,0)
		APBrowseSortBuysArrow:ClearAllPoints()
		APBrowseSortBuysArrow:SetPoint("LEFT","APBrowseSortBuysText","RIGHT",3,0)
		APBrowseSortBuys:SetWidth(col2width)
		APBrowseSortBuys:SetPoint("LEFT","APFrame_sort","LEFT",col2,0)
		APBrowseSortProfitText:SetText(L["Profit"])
		APBrowseSortProfitText:ClearAllPoints()
		APBrowseSortProfitText:SetPoint("RIGHT",0,0)
		APBrowseSortProfitArrow:ClearAllPoints()
		APBrowseSortProfitArrow:SetPoint("RIGHT","APBrowseSortProfitText","LEFT",-3,0)
		APBrowseSortProfit:SetWidth(col3width)
		APBrowseSortProfit:SetPoint("LEFT","APFrame_sort","LEFT",col3,0)
		for i=1, #APlastAuctions[AP_ServerNameIndex] do
			local calc = ((12*i*(-1))+2)
			local n = _G["APList"..i.."Name"]
			local b = _G["APList"..i.."Buys"]
			local p = _G["APList"..i.."Profit"]
			n:SetWidth(col1width)
			n:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col1, calc)
			n:SetTextColor(1, 1, 0, 1) -- yellow

			b:SetWidth(col2width)
			b:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col2, calc)
			b:SetTextColor(1, 1, 1, 1) -- white
			b:SetJustifyH("CENTER")

			p:SetWidth(col3width)
			p:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col3, calc)
		end
	elseif num == 1 then
		local col1 = 10
		local col1width = 270
		local col2 = 280
		local col2width = 90
		local col3 = 370
		local col3width = 100
		setHeadlineTabButton("APFrame_headline_ButtonName", nil)
		setHeadlineTabButton("APFrame_headline_ButtonItem", true)
		setHeadlineTabButton("APFrame_headline_ButtonLastAuctions", nil)

		APFrame_searchAlignFrame:Show()

		APBrowseSortName:Enable()
		APBrowseSortBuys:Enable()
		APBrowseSortProfit:Enable()
		APBrowseSortNameText:SetText(L["Item"])
		APBrowseSortNameText:ClearAllPoints()
		APBrowseSortNameText:SetPoint("LEFT",0,0)
		APBrowseSortNameArrow:ClearAllPoints()
		APBrowseSortNameArrow:SetPoint("LEFT","APBrowseSortNameText","RIGHT",3,0)
		APBrowseSortName:SetWidth(col1width)
		APBrowseSortName:SetPoint("LEFT","APFrame_sort","LEFT",col1,0)
		APBrowseSortBuysText:SetText(L["Auctions"])
		APBrowseSortBuysText:ClearAllPoints()
		APBrowseSortBuysText:SetPoint("CENTER",0,0)
		APBrowseSortBuysArrow:ClearAllPoints()
		APBrowseSortBuysArrow:SetPoint("LEFT","APBrowseSortBuysText","RIGHT",3,0)
		APBrowseSortBuys:SetWidth(col2width)
		APBrowseSortBuys:SetPoint("LEFT","APFrame_sort","LEFT",col2,0)
		APBrowseSortProfitText:SetText(L["Profit"])
		APBrowseSortProfitText:ClearAllPoints()
		APBrowseSortProfitText:SetPoint("RIGHT",0,0)
		APBrowseSortProfitArrow:ClearAllPoints()
		APBrowseSortProfitArrow:SetPoint("RIGHT","APBrowseSortProfitText","LEFT",-3,0)
		APBrowseSortProfit:SetWidth(col3width)
		APBrowseSortProfit:SetPoint("LEFT","APFrame_sort","LEFT",col3,0)
		for i=1, #APlastAuctions[AP_ServerNameIndex] do
			local calc = ((12*i*(-1))+2)
			local n = _G["APList"..i.."Name"]
			local b = _G["APList"..i.."Buys"]
			local p = _G["APList"..i.."Profit"]
			n:SetWidth(col1width)
			n:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col1, calc)
			n:SetTextColor(0.0001, 0.6981, 0.4824, 1) -- green

			b:SetWidth(col2width)
			b:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col2, calc)
			b:SetTextColor(1, 1, 1, 1) -- white
			b:SetJustifyH("CENTER")

			p:SetWidth(col3width)
			p:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col3, calc)
		end
	elseif num == 2 then
		local col1 = 10
		local col1width = 250
		local col2 = 260
		local col2width = 110
		local col3 = 370
		local col3width = 100
		setHeadlineTabButton("APFrame_headline_ButtonName", nil)
		setHeadlineTabButton("APFrame_headline_ButtonItem", nil)
		setHeadlineTabButton("APFrame_headline_ButtonLastAuctions", true)

		APFrame_searchAlignFrame:Hide()

		APBrowseSortName:Disable()
		APBrowseSortBuys:Disable()
		APBrowseSortProfit:Disable()
		APBrowseSortNameText:SetText(L["Item"])
		APBrowseSortNameText:ClearAllPoints()
		APBrowseSortNameText:SetPoint("LEFT",0,0)
		--APBrowseSortNameArrow:ClearAllPoints()
		--APBrowseSortNameArrow:SetPoint("LEFT","APBrowseSortNameText","RIGHT",3,0)
		APBrowseSortName:SetWidth(col1width)
		APBrowseSortName:SetPoint("LEFT","APFrame_sort","LEFT",col1,0)
		APBrowseSortBuysText:SetText(L["Name"])
		APBrowseSortBuysText:ClearAllPoints()
		APBrowseSortBuysText:SetPoint("LEFT",0,0)
		--APBrowseSortBuysArrow:ClearAllPoints()
		--APBrowseSortBuysArrow:SetPoint("LEFT","APBrowseSortBuysText","RIGHT",3,0)
		APBrowseSortBuys:SetWidth(col2width)
		APBrowseSortBuys:SetPoint("LEFT","APFrame_sort","LEFT",col2,0)
		APBrowseSortProfitText:SetText(L["Profit"])
		APBrowseSortProfitText:ClearAllPoints()
		APBrowseSortProfitText:SetPoint("RIGHT",0,0)
		--APBrowseSortProfitArrow:ClearAllPoints()
		--APBrowseSortProfitArrow:SetPoint("RIGHT","APBrowseSortProfitText","LEFT",-3,0)
		APBrowseSortProfit:SetWidth(col3width)
		APBrowseSortProfit:SetPoint("LEFT","APFrame_sort","LEFT",col3,0)
		for i=1, #APlastAuctions[AP_ServerNameIndex] do
			local calc = ((12*i*(-1))+2)
			local n = _G["APList"..i.."Name"]
			local b = _G["APList"..i.."Buys"]
			local p = _G["APList"..i.."Profit"]
			n:SetWidth(col1width)
			n:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col1, calc)
			n:SetTextColor(0.0001, 0.6981, 0.4824, 1) -- green

			b:SetWidth(col2width)
			b:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col2, calc)
			b:SetTextColor(1, 1, 0, 1) -- yellow
			b:SetJustifyH("LEFT")

			p:SetWidth(col3width)
			p:SetPoint("LEFT", "APFrame_ListView", "TOPLEFT", col3, calc)
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:DisplayMode(num)
	if not AP_ServerNameIndex then return end
	AP_sortData = num
	AuctionProfit:UpdateFrame(AP_sortData)
	AuctionProfit:FillList(AP_sortType, AP_sortOrder, AP_sortData)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:TableSortOrder(n)
	if n == 1 then
		if AP_sortType == 0 then
			if     AP_sortOrder == 0 then AuctionProfit:FillList(0, 1, AP_sortData) return
			elseif AP_sortOrder == 1 then AuctionProfit:FillList(0, 0, AP_sortData) return end
		else
			AuctionProfit:FillList(0, 0, AP_sortData) return
		end
	elseif n == 2 then
		if AP_sortType == 1 then
			if     AP_sortOrder == 0 then AuctionProfit:FillList(1, 1, AP_sortData) return
			elseif AP_sortOrder == 1 then AuctionProfit:FillList(1, 0, AP_sortData) return end
		else
			AuctionProfit:FillList(1, 0, AP_sortData) return
		end
	elseif n == 3 then
		if AP_sortType == 2 then
			if     AP_sortOrder == 0 then AuctionProfit:FillList(2, 1, AP_sortData) return
			elseif AP_sortOrder == 1 then AuctionProfit:FillList(2, 0, AP_sortData) return end
		else
			AuctionProfit:FillList(2, 0, AP_sortData) return
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:StatusLineTxt()
	local ItemCount = 0
	local ProfitCount = 0
	local AuctionCount = 0
	for i=1, AP_NameCount do
		local buyer = APbuyer[AP_ServerNameIndex][i]
		local auctions = strReturn(buyerMainDataReturn(buyer),3)
		AuctionCount = AuctionCount + auctions
	end
	for i=1, #APitemName do
		if itemNameServerCheckReturn(APitemName[i],AP_ServerNameIndex) then
			ItemCount = ItemCount + 1
			ProfitCount = ProfitCount + tonumber(itemNameSubDataPartProfitReturn(APitemName[i],AP_ServerNameIndex))
		end
	end
	local txt = "|cff"..colorBlue..L["Server"]..":|r |cff"..colorWhite..APserver[AP_ServerNameIndex].."|r"
	txt = txt.." - |cff"..colorBlue..L["Profit"]..":|r "..moneyFormat(ProfitCount).."\n"
	txt = txt.."|cff"..colorBlue..L["Auctions"]..":|r |cff"..colorWhite..AuctionCount.."|r"
	txt = txt.." - |cff"..colorBlue..L["Buyer"]..":|r |cff"..colorWhite..AP_NameCount.."|r"
	txt = txt.." - |cff"..colorBlue..L["Items"]..":|r |cff"..colorWhite..ItemCount.."|r"

	APFrame_StatusLineTxt:SetText(txt)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:FrameToggle()
	if APFrame:IsShown() then
		APFrame:Hide()
	else
		APFrame:Show()
	end
end

function AuctionProfit:MainFramePos()
	if AuctionProfit_Options.relP then
		APFrame:ClearAllPoints()
		APFrame:SetPoint(AuctionProfit_Options.relP, AuctionProfit_Options.posX, AuctionProfit_Options.posY)
	end
end

function AuctionProfit:MainFrameHide()
	AP_ListData = nil
	PlaySound("igQuestListClose")
	AuctionProfit_InterfaceOptions_GUI:Enable()

	AuctionProfit_InterfaceOptions_GUILeft:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
	AuctionProfit_InterfaceOptions_GUIMiddle:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
	AuctionProfit_InterfaceOptions_GUIRight:SetTexture("Interface\\Buttons\\UI-Panel-Button-Up")
end

function AuctionProfit:MainFrameShow()
	PlaySound("igQuestListOpen")
	AuctionProfit_InterfaceOptions_GUI:Disable()

	AuctionProfit_InterfaceOptions_GUI:SetDisabledTexture(nil)
	AuctionProfit_InterfaceOptions_GUILeft:SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
	AuctionProfit_InterfaceOptions_GUIMiddle:SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
	AuctionProfit_InterfaceOptions_GUIRight:SetTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")

	AuctionProfit:DisplayMode(AP_sortData)
	AuctionProfit:MainFramePos()
end

function AuctionProfit:SaveFramePos()
	local point, relativeTo, relativePoint, x, y = APFrame:GetPoint()
	AuctionProfit_Options.posX = x
	AuctionProfit_Options.posY = y
	AuctionProfit_Options.relP = relativePoint
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:SearchFrameToggle()
	if APFrame_search:IsShown() then
		AuctionProfit:SearchFrameHide()
		AuctionProfit:DisplayMode(AP_sortData)
	else
		AuctionProfit:SearchFrameShow()
	end
end

function AuctionProfit:SearchFrameHide()
	SearchQuery = nil
	APFrame_searchBox:SetText("")
	APFrame_search:Hide()
	APFrame_sort:SetPoint("TOPLEFT", "APFrame_headline", "BOTTOMLEFT", 0, 0)
	AuctionProfit_Options.APwindowSearchShow = false
end

function AuctionProfit:SearchFrameShow()
	APFrame_search:Show()
	APFrame_sort:SetPoint("TOPLEFT", "APFrame_search", "BOTTOMLEFT", 0, 0)
	AuctionProfit_Options.APwindowSearchShow = true
end

function AuctionProfit:Search()
	SearchQuery = APFrame_searchBox:GetText()
	AuctionProfit:DisplayMode(AP_sortData)
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:LineOnEnter(self, name)
	if not AP_ServerNameIndex then return end
	if not AuctionProfit_Options.APwindowTooltipToggle then return end
	if AP_sortData == 0 then
		local PlayerName = _G[name.."Name"]:GetText()
		local TooltipText = AuctionProfit:TooltipSummary(PlayerName, "APwindow")
		if TooltipText then
			GameTooltip:SetOwner(self, AuctionProfit_Options.APwindowTooltipPosAnchor, AuctionProfit_Options.APwindowTooltipPosX, AuctionProfit_Options.APwindowTooltipPosY)
			GameTooltip:SetText("|cff"..colorYellow..PlayerName.."|r\n\n"..TooltipText)
			GameTooltip:Show()
		end
	elseif AP_sortData == 1 then
		local ItemName = _G[name.."Name"]:GetText()
		local TooltipText = AuctionProfit:TooltipItemSummary(ItemName)
		if TooltipText then
			GameTooltip:SetOwner(self, AuctionProfit_Options.APwindowTooltipPosAnchor, AuctionProfit_Options.APwindowTooltipPosX, AuctionProfit_Options.APwindowTooltipPosY)
			GameTooltip:SetText("|cff"..colorGreen..ItemName.."|r\n\n"..TooltipText)
			GameTooltip:Show()
		end
	elseif AP_sortData == 2 then
		local PlayerName = _G[name.."Buys"]:GetText()
		local pos = string_find(PlayerName, " ", 1)
		local PlayerName = string_sub(PlayerName, 1, pos-1)
		local TooltipText = AuctionProfit:TooltipSummary(PlayerName, "APwindow")
		if TooltipText then
			GameTooltip:SetOwner(self, AuctionProfit_Options.APwindowTooltipPosAnchor, AuctionProfit_Options.APwindowTooltipPosX, AuctionProfit_Options.APwindowTooltipPosY)
			GameTooltip:SetText("|cff"..colorYellow..PlayerName.."|r\n\n"..TooltipText)
			GameTooltip:Show()
		end
	end
end

function AuctionProfit:LineOnLeave(name)
--	if not AP_ServerNameIndex then return end
--	if not AuctionProfit_Options.APwindowTooltipToggle then return end
	GameTooltip:Hide()
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:TooltipSummary(PlayerName, WhichTooltip)
	local TooltipText
	-- name loop start
	for i=1, AP_NameCount do
		if AP_NameIndex[i] == PlayerName then
			local LinesCheck = AuctionProfit_Options.PlayerTooltipLines
			if WhichTooltip == "AH" then
				LinesCheck = AuctionProfit_Options.AHTooltipLines
			end
			if WhichTooltip == "APwindow" then
				LinesCheck = AuctionProfit_Options.APwindowTooltipLines
			end
			local temp = {}
			local MainData = APbuyer[AP_ServerNameIndex][i]
			local SubData = buyerSubDataReturn(MainData)
			local BuyerMainData = buyerMainDataReturn(MainData)
			local TotalProfit = strReturn(BuyerMainData,2)
			local TotalAuctions = strReturn(BuyerMainData,3)
			local TooltipTextDataM = ""
			TooltipText = "|cff"..colorBlue..L["Profit"]..":|r "..moneyFormat(tonumber(TotalProfit)).." |cff"..colorWhite.."("..TotalAuctions..")|r"
			if LinesCheck > 0 then

				-- buys loop start
				for j=1, buyerSubDataCountReturn(MainData) do
					local al = string_len(SubData)
					local as = string_find(SubData, ":", 1)
					local as2 = string_find(SubData, "-", 1)
					local r1 = string_sub(SubData, 1, (as2-1))
					local itemname = itemNameReturn(APitemName[tonumber(r1)])
					if string_len(itemname) > AuctionProfit_Options.TooltipItemNameLength then
						itemname = string_sub(itemname, 0, AuctionProfit_Options.TooltipItemNameLength).."..."
					end
					local count = buyerSubDataPartBuysReturn(MainData,tonumber(r1))
					local profit = tonumber(buyerSubDataPartProfitReturn(MainData,tonumber(r1)))
					-- TooltipLineDisplay
					if AuctionProfit_Options.TooltipLineDisplay == 1 then
						TooltipTextDataM = "|cff"..colorGreen..itemname.."|r |cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r "..numMatrix(tostring(profit), 8)..
						                   "#-:-#"..
						                   "|cff"..colorGreen..itemname.."|r |cff"..colorWhite.."("..count..")|r "..moneyFormat(profit)
						table_insert(temp, TooltipTextDataM)
					elseif AuctionProfit_Options.TooltipLineDisplay == 2 then
						TooltipTextDataM = "|cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r |cff"..colorGreen..itemname.."|r "..numMatrix(tostring(profit), 8)..
						                   "#-:-#"..
						                   "|cff"..colorWhite.."("..count..")|r |cff"..colorGreen..itemname.."|r "..moneyFormat(profit)
						table_insert(temp, TooltipTextDataM)
					elseif AuctionProfit_Options.TooltipLineDisplay == 3 then
						TooltipTextDataM = "|cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r "..numMatrix(tostring(profit), 8).." |cff"..colorGreen..itemname.."|r"..
						                   "#-:-#"..
						                   "|cff"..colorWhite.."("..count..")|r "..moneyFormat(profit).." |cff"..colorGreen..itemname.."|r"
						table_insert(temp, TooltipTextDataM)
					elseif AuctionProfit_Options.TooltipLineDisplay == 4 then
						TooltipTextDataM = numMatrix(tostring(profit), 8).." |cff"..colorGreen..itemname.."|r |cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r"..
						                   "#-:-#"..
						                   moneyFormat(profit).." |cff"..colorGreen..itemname.."|r |cff"..colorWhite.."("..count..")|r"
						table_insert(temp, TooltipTextDataM)
					elseif AuctionProfit_Options.TooltipLineDisplay == 5 then
						TooltipTextDataM = numMatrix(tostring(profit), 8).." |cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r |cff"..colorGreen..itemname.."|r"..
						                   "#-:-#"..
						                   moneyFormat(profit).." |cff"..colorWhite.."("..count..")|r |cff"..colorGreen..itemname.."|r"
						table_insert(temp, TooltipTextDataM)
					end
					if as == nil or as == 0 then
						break
					else
						SubData = string_sub(SubData, (as+1), al)
					end
				end
				-- buys loop end
			
				table_sort(temp, function(a,b) return a>b end)

				TooltipText = TooltipText.."\n"
				for k=1, #temp do
					local s = temp[k]
					local pos = string_find(s, "#-:-#", 1)
					local sub = string_sub(s, (pos+5))
					TooltipText = TooltipText..sub.."\n"
					if k == LinesCheck then
						TooltipText = TooltipText.."|cff"..colorLightyellow.."..."..L["more"].."|r"
						break
					end
				end
			end

			table_wipe(temp)

			break
		end
	end
	-- name loop end
	return TooltipText
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:TooltipItemSummary(ItemName)
	local TooltipText
	-- name loop start
	for i=1, AP_ItemCount do
		local ii = AP_ItemIndex[i]
		if string_find(APitemName[ii],ItemName.."#",1,true) then
			local LinesCheck = AuctionProfit_Options.APwindowTooltipLines
			local temp = {}
			local MainData = APitemName[ii]
			local TotalProfit = itemNameSubDataPartProfitReturn(MainData, AP_ServerNameIndex)
			local TotalAuctions = itemNameSubDataPartBuysReturn(MainData, AP_ServerNameIndex)
			local TooltipTextDataM = ""
			TooltipText = "|cff"..colorBlue..L["Profit"]..":|r "..moneyFormat(tonumber(TotalProfit)).." |cff"..colorWhite.."("..TotalAuctions..")|r"

			-- item loop start
			for j=1, AP_NameCount do
				local buyer = APbuyer[AP_ServerNameIndex][j]
				
				if string_find(buyer,ii,1,true) then			
					if string_find(buyer,ii.."-",1,true) then
						local l = string_len(buyer)
						local pos = string_find(buyer,"#",1)
						local a = ":"..string_sub(buyer,pos+1,l)
						local b = ":"..ii.."-"

						if string_find(a,b,1,true) then
							local name = AP_NameIndex[j]
							local count = buyerSubDataPartBuysReturn(buyer,ii)
							local profit = tonumber(buyerSubDataPartProfitReturn(buyer,ii))
							-- TooltipLineDisplay
							if AuctionProfit_Options.TooltipLineDisplay == 1 then
								TooltipTextDataM = "|cff"..colorYellow..name.."|r |cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r "..numMatrix(tostring(profit), 8)..
								                   "#-:-#"..
								                   "|cff"..colorYellow..name.."|r |cff"..colorWhite.."("..count..")|r "..moneyFormat(profit)
								table_insert(temp, TooltipTextDataM)
							elseif AuctionProfit_Options.TooltipLineDisplay == 2 then
								TooltipTextDataM = "|cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r |cff"..colorYellow..name.."|r "..numMatrix(tostring(profit), 8)..
								                   "#-:-#"..
								                   "|cff"..colorWhite.."("..count..")|r |cff"..colorYellow..name.."|r "..moneyFormat(profit)
								table_insert(temp, TooltipTextDataM)
							elseif AuctionProfit_Options.TooltipLineDisplay == 3 then
								TooltipTextDataM = "|cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r "..numMatrix(tostring(profit), 8).." |cff"..colorYellow..name.."|r"..
								                   "#-:-#"..
								                   "|cff"..colorWhite.."("..count..")|r "..moneyFormat(profit).." |cff"..colorYellow..name.."|r"
								table_insert(temp, TooltipTextDataM)
							elseif AuctionProfit_Options.TooltipLineDisplay == 4 then
								TooltipTextDataM = numMatrix(tostring(profit), 8).." |cff"..colorYellow..name.."|r |cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r"..
								                   "#-:-#"..
								                   moneyFormat(profit).." |cff"..colorYellow..name.."|r |cff"..colorWhite.."("..count..")|r"
								table_insert(temp, TooltipTextDataM)
							elseif AuctionProfit_Options.TooltipLineDisplay == 5 then
								TooltipTextDataM = numMatrix(tostring(profit), 8).." |cff"..colorWhite.."("..numMatrix(tostring(count), 8)..")|r |cff"..colorYellow..name.."|r"..
								                   "#-:-#"..
								                   moneyFormat(profit).." |cff"..colorWhite.."("..count..")|r |cff"..colorYellow..name.."|r"
								table_insert(temp, TooltipTextDataM)
							end
						end

					end
				end

			end
			-- item loop end
			
			table_sort(temp, function(a,b) return a>b end)

			TooltipText = TooltipText.."\n"
			for k=1, #temp do
				local s = temp[k]
				local pos = string_find(s, "#-:-#", 1)
				local sub = string_sub(s, (pos+5))
				TooltipText = TooltipText..sub.."\n"
				if k == LinesCheck then
					TooltipText = TooltipText.."|cff"..colorLightyellow.."..."..L["more"].."|r"
					break
				end
			end

			table_wipe(temp)

			break
		end
	end
	-- name loop end
	return TooltipText
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:OnTooltipSetUnit()
	if not AuctionProfit_Options.PlayerTooltipToggle then return end
	if not AP_ServerNameIndex then return end
	if not UnitIsPlayer("mouseover") then return end
	if AuctionProfit_Options.PlayerTooltipOOC and UnitAffectingCombat("player") then return end

	local name, realm = UnitName("mouseover")
	if not name then return end
	if name == UNKNOWNOBJECT then return end
	if realm == "" then return end
	if realm then return end
	local TooltipText = AuctionProfit:TooltipSummary(name, "Player")
	if TooltipText then
		GameTooltip:AddLine(TooltipText, 1, 1, 1)
		GameTooltip:Show()
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
function AuctionProfit:AuctionFrameBrowse_Update()
	if not AuctionProfit_Options.AHBrowseTabProfit then return end
	local offset = BrowseScrollFrame.offset
	for i=1, NUM_BROWSE_TO_DISPLAY do
		local owner = select(12, GetAuctionItemInfo("list", offset + i))

		if owner then
			local profit
			for j=1, AP_NameCount do
				if AP_NameIndex[j] == owner then
					local buyerData = buyerMainDataReturn(APbuyer[AP_ServerNameIndex][j])
					profit = moneyFormat( tonumber( strReturn(buyerData,2) ) )
					break
				end
			end

			-- cleanup
			if i > NUM_BROWSE_TO_DISPLAY then i = NUM_BROWSE_TO_DISPLAY end
			if i < 1 then i = 1 end
			-- cleanup

			local frame = _G["APFrame_AHBrowseTabFrame"..i]
			if profit then
				local text = _G["APFrame_AHBrowseTabFrameText"..i]
				local texture = _G["APFrame_AHBrowseTabFrameTexture"..i]
				text:SetText(profit)
				local w = text:GetStringWidth()
				frame:SetWidth(w+2)
				texture:SetWidth(w+2)
				frame:Show()
			else
				frame:Hide()
			end
		end
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

-- ---------------------------------------------------------------------------------------------------------------------
local function AuctionProfit_GameTooltip_AuctionOnEnter(self, ...)
	if not AuctionProfit_Options.AHTooltipToggle then return end
	local id = self:GetID() + BrowseScrollFrame.offset -- FauxScrollFrame_GetOffset(BrowseScrollFrame)
	local owner = select(12, GetAuctionItemInfo("list", id))
	if owner then
		local TooltipText = AuctionProfit:TooltipSummary(owner, "AH")
		if TooltipText then
			GameTooltip:SetOwner(self, AuctionProfit_Options.AHTooltipPosAnchor, AuctionProfit_Options.AHTooltipPosX, AuctionProfit_Options.AHTooltipPosY)
			GameTooltip:SetText(owner.."\n\n"..TooltipText)
			GameTooltip:Show()
		end
	end
end

local function AuctionProfit_GameTooltip_AuctionOnLeave()
	GameTooltip:Hide()
end
-- ---------------------------------------------------------------------------------------------------------------------



-- ---------------------------------------------------------------------------------------------------------------------
function OnEvent(self, event, arg1)
	if event == "AUCTION_HOUSE_SHOW" then
		if not APFrame_AHBrowseTabFrame1 then
			for i=1, NUM_BROWSE_TO_DISPLAY do
				local framename   = "APFrame_AHBrowseTabFrame"..i
				local textname    = "APFrame_AHBrowseTabFrameText"..i
				local texturename = "APFrame_AHBrowseTabFrameTexture"..i
				local parent = _G["BrowseButton"..i]
				local point = _G["BrowseButton"..i.."HighBidder"]
				local frame = CreateFrame("Frame", framename, parent)
				frame:ClearAllPoints()
				frame:SetPoint("BOTTOM", point, "BOTTOM", 0, -2)
				frame:SetWidth(100)
				frame:SetHeight(12)
				frame:Hide()
				local texture = frame:CreateTexture(texturename, "DIALOG")
				texture:SetPoint("CENTER", framename, "CENTER", 0, 0)
				texture:SetTexture(0, 0, 0, 0.3)
				texture:SetWidth(100)
				texture:SetHeight(12)
				local text = frame:CreateFontString(textname, "DIALOG", "GameFontNormalSmall")
				text:SetPoint("CENTER", framename, "CENTER", 0, 2)

				_G["BrowseButton"..i]:SetScript("OnEnter", AuctionProfit_GameTooltip_AuctionOnEnter) --need check
				_G["BrowseButton"..i]:SetScript("OnLeave", AuctionProfit_GameTooltip_AuctionOnLeave) --need check
			end
			hooksecurefunc("AuctionFrameBrowse_Update", AuctionProfit.AuctionFrameBrowse_Update)
		end
		AuctionProfit:UnregisterEvent("AUCTION_HOUSE_SHOW")
	elseif event == "ADDON_LOADED" then
		if arg1 == "AuctionProfit" then
			AuctionProfit:InitOptions()
			AuctionProfit:CreateInterfaceOptions()
			AuctionProfit:DataUpdate()
			AuctionProfit:InitFrame()

			tinsert(UISpecialFrames,"APFrame")

			GameTooltip:HookScript("OnTooltipSetUnit", AuctionProfit.OnTooltipSetUnit)
			hooksecurefunc("TakeInboxMoney", AuctionProfit_TakeInboxMoney)
			hooksecurefunc("InboxFrame_OnClick", AuctionProfit_OopsIDidItAgain)
			OpenMailFrame:HookScript("OnHide", AuctionProfit_OopsIDidItAgain)
			
			AuctionProfit:UnregisterEvent("ADDON_LOADED")
		end
	elseif event == "PLAYER_LOGIN" then
		AuctionProfit:LDBcheck()
		AuctionProfit:UnregisterEvent("PLAYER_LOGIN")
	end
end
-- ---------------------------------------------------------------------------------------------------------------------

AuctionProfit:RegisterEvent("PLAYER_LOGIN")
AuctionProfit:RegisterEvent("ADDON_LOADED")
AuctionProfit:RegisterEvent("AUCTION_HOUSE_SHOW")
AuctionProfit:SetScript("OnEvent", OnEvent) -- start the Mystical Machine